/*
 * Copyright 2014, 2015 Peter Hansen and İnanç Seylan.
 * 
 * This file is part of Grind.
 * 
 * Grind is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Grind is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Grind.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.unibremen.informatik.tdki.grind.gammaconstruction;

import java.util.HashSet;
import java.util.Iterator;
import java.util.Objects;
import java.util.Set;
import org.jgrapht.DirectedGraph;
import org.jgrapht.alg.CycleDetector;
import org.semanticweb.owlapi.model.OWLClass;

/**
 *
 * @author hansen
 */
public class Gamma {

    Set<GammaTuple> gammaTuples;
    OWLClass startConcept;
    GammaGraphCreator gammaGraphCreator;
    CycleDetector gammaCycleDetector;
    DirectedGraph gammaGraph;

    public Gamma(OWLClass startConcept) {
        gammaTuples = new HashSet<>();
        this.startConcept = startConcept;
    }

    public Gamma(GammaTuple firstTuple, OWLClass startConcept) {
        this(startConcept);
        this.gammaTuples.add(firstTuple);
    }

    public Gamma(Set<GammaTuple> setOfTuples, OWLClass startConcept) {
        this(startConcept);
        for (GammaTuple tuple : setOfTuples) {
            this.gammaTuples.add(tuple);
        }
    }

    public void addTuple(GammaTuple tuple) {
        gammaTuples.add(tuple);
    }

    public int size() {
        return gammaTuples.size();
    }

    public Set<GammaTuple> getGammaTuples() {
        return this.gammaTuples;
    }

    public DirectedGraph getGammaGraph() {
        return gammaGraph;
    }

    public void createGammaGraph() {
        gammaGraphCreator = new GammaGraphCreator(this, startConcept);
        gammaGraph = gammaGraphCreator.getGraph();
        gammaCycleDetector = new CycleDetector(gammaGraph);
    }

    public boolean isCyclic() {
        return gammaCycleDetector.detectCycles();
    }

    public Set<GammaTuple> getTuplesOnCycle() {
        return gammaCycleDetector.findCycles();
    }

    public int numberOfTuplesOnCycle() {
        return gammaCycleDetector.findCycles().size();
    }

    @Override
    public String toString() {
        String result = "{\n";
        for (Iterator<GammaTuple> tupleIter = this.gammaTuples.iterator(); tupleIter.hasNext();) {
            result += "  " + tupleIter.next().toString();
            if (tupleIter.hasNext()) {
                result += ",\n";
            }
        }
        result += "\n}";
        return result;
    }

    @Override
    public boolean equals(Object obj) {
        if (obj == null) {
            return false;
        }
        if (getClass() != obj.getClass()) {
            return false;
        }
        final Gamma other = (Gamma) obj;
        if (!Objects.equals(this.gammaTuples, other.gammaTuples)) {
            return false;
        }
        return true;
    }
}
