/*
 * Copyright 2014, 2015 Peter Hansen and İnanç Seylan.
 * 
 * This file is part of Grind.
 * 
 * Grind is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Grind is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Grind.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.unibremen.informatik.tdki.grind.omegaconstruction;

import java.util.Iterator;
import java.util.Objects;
import java.util.Set;
import java.util.SortedSet;
import org.semanticweb.owlapi.model.OWLClassExpression;
import org.semanticweb.owlapi.model.OWLObjectSomeValuesFrom;

/**
 *
 * @author hansen
 */
public final class OmegaTuple {

    private final OWLClassExpression concept;
    private final SortedSet<OWLClassExpression> setOfConcepts;
    private final Set<OWLClassExpression> con;
    private final OWLObjectSomeValuesFrom successor;
    private final Set<OWLClassExpression> xcon;

    public OmegaTuple(OWLClassExpression concept, SortedSet<OWLClassExpression> setofConcepts,
            Set<OWLClassExpression> con, OWLObjectSomeValuesFrom successor,
            Set<OWLClassExpression> xcon) {
        this.concept = concept;
        this.setOfConcepts = setofConcepts;
        this.con = con;
        this.successor = successor;
        this.xcon = xcon;
    }

    public OWLClassExpression getConcept() {
        return concept;
    }

    public SortedSet<OWLClassExpression> getSetOfConcepts() {
        return setOfConcepts;
    }

    public Set<OWLClassExpression> getCon() {
        return con;
    }

    public OWLObjectSomeValuesFrom getSuccessor() {
        return successor;
    }

    public Set<OWLClassExpression> getXCon() {
        return xcon;
    }

    public int getNumberOfExInS() {
        int result = 0;
        for (OWLClassExpression c : this.setOfConcepts) {
            if (c instanceof OWLObjectSomeValuesFrom) {
                result++;
            }
        }
        return result;
    }

    @Override
    public String toString() {
        String result = "(";

        if (this.concept == null) {
            result += "--";
        } else {
            result += this.getConcept().toString();
        }
        result += ", {";

        if (this.setOfConcepts == null) {
            result += "--";
        } else {
            for (Iterator<OWLClassExpression> it = this.getSetOfConcepts().iterator();
                    it.hasNext();) {
                result += it.next().toString();
                if (it.hasNext()) {
                    result += ", ";
                }
            }
        }
        result += "}, {";

        if (this.con == null) {
            result += "--";
        } else {

            for (Iterator<OWLClassExpression> it = this.con.iterator();
                    it.hasNext();) {
                result += it.next().toString();
                if (it.hasNext()) {
                    result += ", ";
                }
            }
        }

        if (this.successor == null) {
            result += "}, --, ";
        } else {
            result += "}, " + this.successor + ", ";
        }

        if (this.xcon == null) {
            result += "--";
        } else {
            result += "{";
            for (Iterator<OWLClassExpression> it = this.xcon.iterator();
                    it.hasNext();) {
                result += it.next().toString();
                if (it.hasNext()) {
                    result += ", ";
                }
            }
            result += "}";
        }
        result += ")";
        return result;
    }

    private volatile int hash;

    @Override
    public int hashCode() {
        if (hash == 0) {
            hash = 3;
            hash = 71 * hash + Objects.hashCode(this.getConcept());
            hash = 71 * hash + Objects.hashCode(this.getSetOfConcepts());
            hash = 71 * hash + Objects.hashCode(this.con);
            hash = 71 * hash + Objects.hashCode(this.successor);
            hash = 71 * hash + Objects.hashCode(this.xcon);
        }
        return hash;
    }

    @Override
    public boolean equals(Object obj) {
        if (obj == null) {
            return false;
        }
        if (getClass() != obj.getClass()) {
            return false;
        }
        final OmegaTuple other = (OmegaTuple) obj;
        if (!Objects.equals(this.getConcept(), other.getConcept())) {
            return false;
        }
        if (!Objects.equals(this.getSetOfConcepts(), other.getSetOfConcepts())) {
            return false;
        }
        if (!Objects.equals(this.con, other.con)) {
            return false;
        }
        if (!Objects.equals(this.successor, other.successor)) {
            return false;
        }
        if (!Objects.equals(this.xcon, other.xcon)) {
            return false;
        }
        return true;
    }

}
