/*
 * Copyright 2014, 2015 Peter Hansen and İnanç Seylan.
 * 
 * This file is part of Grind.
 * 
 * Grind is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Grind is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Grind.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.unibremen.informatik.tdki.grind.utilities;

import java.util.Arrays;
import java.util.Set;
import java.util.TreeSet;
import org.semanticweb.owlapi.model.IRI;

/**
 *
 * @author hansen
 */
public class StringTools {

    /**
     * Delete duplicate lines. This is used to simplify a rewriting obtained
     * from Omega.
     *
     * @param lines
     * @return
     */
    public static String uniq(String lines) {
        Set<String> resultingSet = new TreeSet<>();
        String[] splitLines = lines.split("\n");
        resultingSet.addAll(Arrays.asList(splitLines));
        String result = "";
        for (String line : resultingSet) {
            result += line + "\n";
        }
        // Delete final "\n" from result:
        result = result.substring(0, result.length() - 1);
        return result;
    }

    /**
     * Sort lines in a string. Mostly for debugging.
     *
     * @param <V>
     */
    public static String sort(String lines) {
        String[] splitLines = lines.split("\n");
        Arrays.sort(splitLines);
        String result = "";
        for (String line : splitLines) {
            result += line + "\n";
        }
        return result;
    }

    /**
     *  Shorten output by replacing OWL stuff, mostly for debugging.
     * @param input
     * @return 
     */
    public static String shorten(Object input) {
        return input.toString()
                .replace("<", "")
                .replace(">", "")
                .replace("ObjectIntersectionOf", "and")
                .replace("ObjectSomeValuesFrom", "ex");
    }

    @Deprecated
    public static String getFragment(IRI iri) {
        String result;
        if (iri.toString().contains("#")) {
            result = iri.toString().split("#")[1];
        } else {
            String[] tokens = iri.toString().split("/");
            result = tokens[tokens.length - 1];
        }
        return result;
    }

}
