/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package de.unibremen.informatik.tdki.grind2.control;

import de.unibremen.informatik.tdki.grind2.control.rewriters.AQRewriter;
import de.unibremen.informatik.tdki.grind2.control.rewriters.ConCQRewriter;
import de.unibremen.informatik.tdki.grind2.control.rewriters.QueryRewriter;
import de.unibremen.informatik.tdki.grind2.control.rewriters.RootedCQRewriter;
import de.unibremen.informatik.tdki.grind2.control.rewriters.TqCQRewriter;
import de.unibremen.informatik.tdki.grind2.context.CQContext;
import de.unibremen.informatik.tdki.grind2.cq.CQ;
import de.unibremen.informatik.tdki.grind2.cq.CQException;
import de.unibremen.informatik.tdki.grind2.datalog.DatalogProgram;
import de.unibremen.informatik.tdki.grind2.syntax.CQParser;
import de.unibremen.informatik.tdki.grind2.syntax.SyntaxException;
import de.unibremen.informatik.tdki.grind2.utilities.OutputUtils;
import de.unibremen.informatik.tdki.grind2.utilities.Signature;
import java.io.IOException;
import java.util.Set;
import org.semanticweb.owlapi.model.OWLAxiom;
import org.semanticweb.owlapi.model.OWLOntologyCreationException;

/**
 *
 * @author hansen
 */
public class OntPlusQueryRunner {

    private final Set<OWLAxiom> axioms;
    private final String ontString;
    private final Signature sigma;

    public OntPlusQueryRunner(Set<OWLAxiom> axioms, String ontString) throws OWLOntologyCreationException {
        this.axioms = axioms;
        this.sigma = Signature.getSignatureForAxioms(axioms);
        this.ontString = ontString;
    }

    public void runAdhocquery(String adhocqueryString) throws IOException, CQException, Exception {
        CQParser parser = new CQParser(sigma);
        parser.readLine(adhocqueryString);
        this.processQueries(parser);
    }

    public void runQueryfile(String queryfileString) throws Exception {
        CQParser parser = new CQParser(sigma);
        parser.readFile(queryfileString);
        this.processQueries(parser);
    }

    private void processQueries(CQParser parser) throws SyntaxException, CQException, Exception {
        int queryNumber = 1;
        for (CQ q : parser.getListOfCQs()) {
            if (CQContext.getINSTANCE().isQueryfile()) {
                System.out.println("\nProcessing query " + queryNumber + ": " + q);
            }
            runQuery(q, queryNumber);
            queryNumber++;
        }
    }

    private int runQuery(CQ q, int queryNumber) throws CQException, Exception {
        long t = System.currentTimeMillis();
        String shortname = ontString + queryNumber;
        QueryRewriter rewr = null;
        CQ p = null;

        int querytype = q.getType();
        switch (querytype) {
            case CQ.ROOTEDCQ: {
                if (CQContext.getINSTANCE().isVerbose()) {
                    System.out.println("Detected as RootedCQ.");
                }
                rewr = new RootedCQRewriter(axioms);
                break;
            }
            case CQ.TQCQ: {
                if (CQContext.getINSTANCE().isVerbose()) {
                    System.out.println("Detected as TqCQ.");
                }
                rewr = new TqCQRewriter(axioms);
                break;
            }
            case CQ.AVARCQ: {
                if (CQContext.getINSTANCE().isVerbose()) {
                    System.out.println("Detected as AvarCQ.");
                }
                rewr = new TqCQRewriter(axioms);
                break;
            }
            case CQ.CONCQ: {
                if (CQContext.getINSTANCE().isVerbose()) {
                    System.out.println("Detected as ConCQ.");
                }
                rewr = new ConCQRewriter(axioms);
                break;
            }
            case CQ.AQ: {
                if (CQContext.getINSTANCE().isVerbose()) {
                    System.out.println("Detected as AQ.");
                }
                rewr = new AQRewriter(axioms);
                break;
            }
            default: {
                System.err.println("Cannot process this type of CQ (neither AQ/TreeCQ/AvarCQ/TqCQ/RootedCQ)! Aborting.");
                System.exit(-1);
            }
        }

        int retval = rewr.rewrite(q);

        switch (retval) {
            case 1: {
                System.out.println("Query " + q + " is not FO-rewritable (Omega CYCLIC)!");
                return retval;
            }
            case -1: {
                System.out.println("TIMEOUT Gamma!");
                return retval;
            }
            case -2: {
                System.out.println("TIMEOUT Omega!");
                return retval;
            }
            default: {
                throw new CQException("[" + this.getClass().getSimpleName()
                        + "] Unexpected return value: " + retval);
            }
            case 0: {

                DatalogProgram dlogRewriting = rewr.getResultingProgram();
                if (CQContext.getINSTANCE().isVerbose()) {
                    System.out.println("Non-verbose output only the following:");
                    System.out.println("/--------------------------------------------------------------------------");
                }
                System.out.println(OutputUtils.sortMulitineString(dlogRewriting.toString()));
                if (CQContext.getINSTANCE().isVerbose()) {
                    System.out.println("\\--------------------------------------------------------------------------");
                    System.out.println("End of non-verbose output.");
                }

                if (CQContext.getINSTANCE().isVerbose()) {
                    long totalTime = System.currentTimeMillis() - t;
                    System.out.println("### Computation of query " + shortname);
                    System.out.println("### took " + totalTime + " ms.");
                    System.out.println("#### (" + shortname + ", " + (((double) totalTime) / 1000) + ")");
                }
                return retval;
            }
        }
    }
}
