/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package de.unibremen.informatik.tdki.grind2.control.rewriters;

import com.google.common.collect.Sets;
import de.unibremen.informatik.tdki.grind.control.GammaAndOmegaCaller;
import de.unibremen.informatik.tdki.grind2.cq.CQ;
import de.unibremen.informatik.tdki.grind2.cq.CQException;
import de.unibremen.informatik.tdki.grind2.cq.ConCQ;
import de.unibremen.informatik.tdki.grind2.datalog.DatalogProgram;
import de.unibremen.informatik.tdki.grind2.datalog.DatalogRule;
import de.unibremen.informatik.tdki.grind2.datalog.components.DatalogAtom;
import de.unibremen.informatik.tdki.grind2.datalog.components.DatalogClassAtom;
import de.unibremen.informatik.tdki.grind2.datalog.components.DatalogQueryAtom;
import de.unibremen.informatik.tdki.grind2.datalog.components.DatalogVariable;
import java.util.HashSet;
import java.util.Set;
import org.semanticweb.owlapi.apibinding.OWLManager;
import org.semanticweb.owlapi.model.OWLAxiom;
import org.semanticweb.owlapi.model.OWLClass;
import org.semanticweb.owlapi.model.OWLOntology;

/**
 *
 * @author hansen
 */
public class ConCQRewriter extends QueryRewriter {

    public ConCQRewriter(Set<OWLAxiom> axioms) {
        super(axioms);
    }

    @Override
    public int rewrite(CQ qOrig) throws CQException, Exception {
        ConCQ q0 = qOrig.asConCQ();

        // We get one new CI:
        OWLClass freshConceptname = q0.getFreshConceptname();
        OWLAxiom newCI = OWLManager.getOWLDataFactory().getOWLSubClassOfAxiom(q0.getConcept(), freshConceptname);

        Set<OWLAxiom> axioms = new HashSet<>(this.getAxioms());
        axioms.add(newCI);

        OWLOntology ont = OWLManager.createOWLOntologyManager().createOntology(axioms);
        GammaAndOmegaCaller caller = new GammaAndOmegaCaller(ont);

        int retval = caller.runSingleConceptname(freshConceptname);
        if (retval == 0) {
            DatalogProgram resultingProgram2 = caller.getResultingProgram();

            // Have to replace A_C0 atoms for Q atoms:
            Set<DatalogRule> resultRules = resultingProgram2.getSetOfRules();
            Set<DatalogRule> newRuleSet = new HashSet<>();
            for (DatalogRule rule : resultRules) {
                DatalogAtom head = rule.getHeadAtom();
                if (head instanceof DatalogClassAtom) {
                    DatalogClassAtom clsHead = (DatalogClassAtom) head;
                    DatalogVariable x = clsHead.getX();
                    String conceptString = clsHead.getConcept().toString();
                    if (conceptString.contains("A_C0")) {
                        DatalogAtom replaceHeadAtom = new DatalogQueryAtom(Sets.newHashSet(x));
                        DatalogRule newRule = new DatalogRule(replaceHeadAtom, rule.getBody());
                        newRuleSet.add(newRule);
                    } else {
                        newRuleSet.add(rule);
                    }
                } else {
                    throw new CQException("[" + QueryRewriter.class.getSimpleName() + "] DatalogRoleAtom in head of rule " + rule + "!");
                }
            }

            this.resultingProgram = new DatalogProgram(newRuleSet);
        }
        return retval;
    }

}
