/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package de.unibremen.informatik.tdki.grind2.control.rewriters;

import com.google.common.collect.Sets;
import de.unibremen.informatik.tdki.grind.control.GammaAndOmegaCaller;
import de.unibremen.informatik.tdki.grind.utilities.ELUtils;
import de.unibremen.informatik.tdki.grind2.context.CQContext;
import de.unibremen.informatik.tdki.grind2.cq.AvarCQ;
import de.unibremen.informatik.tdki.grind2.cq.CQ;
import de.unibremen.informatik.tdki.grind2.cq.CQException;
import de.unibremen.informatik.tdki.grind2.cq.TqCQ;
import de.unibremen.informatik.tdki.grind2.cq.components.Atom;
import de.unibremen.informatik.tdki.grind2.cq.components.ClassAtom;
import de.unibremen.informatik.tdki.grind2.cq.components.RoleAtom;
import de.unibremen.informatik.tdki.grind2.cq.components.Variable;
import de.unibremen.informatik.tdki.grind2.datalog.DatalogProgram;
import de.unibremen.informatik.tdki.grind2.datalog.DatalogRule;
import de.unibremen.informatik.tdki.grind2.datalog.components.DatalogAtom;
import de.unibremen.informatik.tdki.grind2.datalog.components.DatalogClassAtom;
import de.unibremen.informatik.tdki.grind2.datalog.components.DatalogQueryAtom;
import de.unibremen.informatik.tdki.grind2.datalog.components.DatalogRoleAtom;
import de.unibremen.informatik.tdki.grind2.datalog.components.DatalogVariable;
import de.unibremen.informatik.tdki.grind2.syntax.Annotations;
import de.unibremen.informatik.tdki.grind2.utilities.OntologyUtils;
import de.unibremen.informatik.tdki.grind2.utilities.OutputUtils;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;
import org.semanticweb.owlapi.apibinding.OWLManager;
import org.semanticweb.owlapi.model.OWLAxiom;
import org.semanticweb.owlapi.model.OWLClass;
import org.semanticweb.owlapi.model.OWLClassExpression;
import org.semanticweb.owlapi.model.OWLObjectProperty;
import org.semanticweb.owlapi.model.OWLObjectPropertyExpression;
import org.semanticweb.owlapi.model.OWLObjectSomeValuesFrom;
import org.semanticweb.owlapi.model.OWLOntology;
import org.semanticweb.owlapi.model.OWLOntologyCreationException;
import org.semanticweb.owlapi.model.OWLSubClassOfAxiom;

/**
 *
 * @author hansen
 */
public class TqCQRewriter extends QueryRewriter {

    public TqCQRewriter(Set<OWLAxiom> axioms) {
        super(axioms);
    }

    Set<DatalogRoleAtom> coreDatalogRoleAtoms;
    Set<DatalogVariable> coreDatalogVariables;

    @Override
    public int rewrite(CQ qOrig) throws OWLOntologyCreationException, Exception, CQException {
        TqCQ q0 = qOrig.asTqCQ();

        // Need this in opti and full red:
        this.coreDatalogRoleAtoms = q0.getCoreDatalogRoleAtoms();
        this.coreDatalogVariables = q0.getCoreDatalogVariables();

        //
        // Reduction to AvarCQ (trees away, use A_C conceptnames):
        //
        // Get set of CNs to test:
        AvarCQ avarq0 = q0.getReducedAvarCQ();
        // Get Map x -> A_Cx:
        Map<Variable, OWLClass> var2newCNmap = q0.getRoot2ACmap();
        // We also get new CIs, for  C(x)  as  A_C42(x):
        Set<OWLAxiom> onlyTreesAwayAxioms = q0.getNewAxioms();
        Set<OWLAxiom> treesAwayAxioms = Sets.union(this.getAxioms(), onlyTreesAwayAxioms);

        //
        // Optimization:
        //
        if (CQContext.getINSTANCE().isVerbose()) {
            System.out.println("tqCQ: Testing FO-rewritability of CNs (optimization)...");
        }

        int optiRetval = this.rewriteAllCNsFromMapOrFail(treesAwayAxioms, var2newCNmap);
        switch (optiRetval) {
            case 0: {
                if (CQContext.getINSTANCE().isVerbose()) {
                    System.out.println("(tqCQ: FO-rewritable by optimization)");
                }
                addQFromAC0AC1AC2rule(var2newCNmap);
                // this.resultingProgram holds Datalog program!
                return 0;
            }
            case -1: {
                if (CQContext.getINSTANCE().isVerbose()) {
                    System.out.println("TIMEOUT Gamma in optimization phase. Aborting.");
                }
                return optiRetval;
            }
            case -2: {
                if (CQContext.getINSTANCE().isVerbose()) {
                    System.out.println("TIMEOUT Omega in optimization phase. Aborting.");
                }
                return optiRetval;
            }
            case 1: {
                if (CQContext.getINSTANCE().isVerbose()) {
                    System.out.println("tqCQ Optimization found non-FOrewr concept name, going into full reduction.");
                }

                //
                // Full reduction:
                //
                Set<OWLAxiom> redAxioms = avarRewriteOntology(avarq0, treesAwayAxioms);

                OWLOntology redOnt = OWLManager.createOWLOntologyManager().createOntology(redAxioms);
                GammaAndOmegaCaller caller = new GammaAndOmegaCaller(redOnt);

                int retval = caller.runSingleConceptname(CQContext.getINSTANCE().getClsN());

                if (retval != 0) {
                    return retval;
                } else {
                    DatalogProgram program = caller.getResultingProgram();
                    if (CQContext.getINSTANCE().isVerbose()) {
                        System.out.println("--- resultingProgram:");
                        System.out.println(program);
                    }

                    this.resultingProgram = OntologyUtils.replaceNbyQReplaceAnnotationsAndAddCore(program,
                            this.coreDatalogRoleAtoms, this.coreDatalogVariables);

                    if (CQContext.getINSTANCE().isVerbose()) {
                        System.out.println("--- Translated resultingProgram:");
                        System.out.println(this.resultingProgram);
                    }

//                    adaptRewritingQForN(var2newCNmap);
//
//                    if (CQContext.getINSTANCE().isVerbose()) {
//                        System.out.println("--- Adapted resultingProgram:");
//                        System.out.println(this.resultingProgram);
//                    }
                    return retval;
                }
            }
            default: {
                return -99;
            }
        }
    }

    public int rewriteAllCNsFromMapOrFail(Set<OWLAxiom> axioms, Map<Variable, OWLClass> var2newCNmap) throws CQException, Exception {
        this.resultingProgram = new DatalogProgram();

        OWLOntology ont = OWLManager.createOWLOntologyManager().createOntology(axioms);
        GammaAndOmegaCaller caller = new GammaAndOmegaCaller(ont);

        for (OWLClass cls : var2newCNmap.values()) {
            if (CQContext.getINSTANCE().isVerbose()) {
                System.out.println("Rewriting one CN: " + cls);
            }
            int retval = caller.runSingleConceptname(cls);
            // 0 FOyes, 1 FOno, -1 GammaTO, -2 OmegaTO.
            if (retval != 0) {
                return retval; // FAIL!
            }
            DatalogProgram programForCls = caller.getResultingProgram();
            if (CQContext.getINSTANCE().isVerbose()) {
                System.out.println("Intermediate program for " + cls + ":");
                System.out.println(OutputUtils.sortMulitineString(programForCls.toString()));
            }

            this.resultingProgram.getSetOfRules().addAll(programForCls.getSetOfRules());
        }
        
        adaptRewritingQForN(var2newCNmap);

        if (CQContext.getINSTANCE().isVerbose()) {
            System.out.println("Complete resultingProgram:");
            System.out.println(OutputUtils.sortMulitineString(resultingProgram.toString()));
        }

        return 0; // retval: all Ok.
    }

    private void addQFromAC0AC1AC2rule(Map<Variable, OWLClass> var2newCNmap) {
        // Adapt the returned rewriting:
        // It should start with: 
        //   Q(?0, ?1, ?2)  <-  A_C0(?0), A_C1(?1), A_C2(?2),   r(?0,?1)... [and other core roles]
        //
        // For A_C0(?0), we have a rewriting in programForCls.
        // We need the first rule, as above:
        //   Q(?0, ?1, ?2)  <-  A_C0(?0), A_C1(?1), A_C2(?2),   r(?0,?1)... [and other core roles]
        DatalogAtom headAtom = new DatalogQueryAtom(this.coreDatalogVariables);
        Set<DatalogAtom> bodyAtoms = new HashSet<>(this.coreDatalogRoleAtoms);

        // Put Class atoms, e.g. A_C0(?0), A_C1(?1), A_C2(?2):
        for (Variable x : var2newCNmap.keySet()) {
            OWLClass clsACx = var2newCNmap.get(x);
            int indexX = x.getIndices().hashCode();
            DatalogVariable xDVar = new DatalogVariable(indexX);
            DatalogClassAtom dCat = new DatalogClassAtom(clsACx, xDVar);

            bodyAtoms.add(dCat);
        }
        DatalogRule firstRule = new DatalogRule(headAtom, bodyAtoms);
        this.resultingProgram.getSetOfRules().add(firstRule);
    }

    private void adaptRewritingQForN(Map<Variable, OWLClass> var2newCNmap) {

        // N(?0)  will be  Q(?0, ?1, ?2)        
        DatalogRule ruleN = null;
        DatalogRule ruleQ = null;
        for (DatalogRule rule : this.resultingProgram.getSetOfRules()) {
            DatalogAtom head = rule.getHeadAtom();
            if (head instanceof DatalogClassAtom) {
                DatalogClassAtom dcHead = (DatalogClassAtom) head;
                if (dcHead.getConcept().equals(CQContext.getINSTANCE().getClsN())) {
                    ruleN = rule;
                    DatalogAtom headQ = new DatalogQueryAtom(this.coreDatalogVariables);
                    ruleQ = new DatalogRule(headQ, rule.getBody());
                }

            }
        }
        if (this.resultingProgram.getSetOfRules().remove(ruleN)) {
            this.resultingProgram.getSetOfRules().add(ruleQ);
        }
    }

    /**
     * Performs avar reduction. Generates all rules of forms (r1), (r2), and
     * (r3).
     *
     * @param q0
     * @param axioms
     * @return
     * @throws OWLOntologyCreationException
     * @throws de.unibremen.informatik.tdki.grind2.cq.CQException
     */
    public static Set<OWLAxiom> avarRewriteOntology(AvarCQ q0, Set<OWLAxiom> axioms) throws OWLOntologyCreationException, CQException {
        Set<OWLAxiom> setOfNewCIs = new HashSet<>(axioms);
        // For every old CI:
        for (OWLAxiom ax : axioms) {
            OWLSubClassOfAxiom ci = (OWLSubClassOfAxiom) ax;
            OWLClassExpression lhs = ci.getSubClass();
            OWLClassExpression rhs = ci.getSuperClass();
            // (r1) To generate new one with A^x ... A^x_exR.C => A^x ... exR^x.C
            //      (lhs new CNs for existentials, rhs only annotated R),
            //      work on tlcs:
            Set<OWLClassExpression> oldLhsTlcs = lhs.asConjunctSet();
            Set<OWLClassExpression> oldRhsTlcs = rhs.asConjunctSet();
            // (r2)(r3) At the same time, break lhs down into subconcepts:
            Set<OWLClassExpression> setOfSubs = ELUtils.getSubconcepts(lhs);
            Set<OWLObjectSomeValuesFrom> setOfExRC = new HashSet<>();
            for (OWLClassExpression sub : setOfSubs) {
                if (sub instanceof OWLObjectSomeValuesFrom) {
                    OWLObjectSomeValuesFrom exRC = (OWLObjectSomeValuesFrom) sub;
                    setOfExRC.add(exRC);
                }
            }
            for (Variable x : q0.getVars()) {
                // (r1)
                Set<OWLClassExpression> newLhsTlcs = Annotations.lhsAnnotation(oldLhsTlcs, x);
                Set<OWLClassExpression> newRhsTlcs = Annotations.rhsAnnotation(oldRhsTlcs, x);
                OWLClassExpression newRhs = OntologyUtils.getClsOrIntersection(newRhsTlcs);
                OWLClassExpression newLhs = OntologyUtils.getClsOrIntersection(newLhsTlcs);
                OWLSubClassOfAxiom newCI = OWLManager.getOWLDataFactory().getOWLSubClassOfAxiom(newLhs, newRhs);
                setOfNewCIs.add(newCI);
                // (r2), (r3)
                for (OWLObjectSomeValuesFrom exRC : setOfExRC) {
                    // (r2)
                    OWLObjectSomeValuesFrom newExRC = CQContext.getINSTANCE().annotateExist(exRC, x);
                    OWLClass newSpurious = CQContext.getINSTANCE().annotateSpurious(exRC, x);
                    OWLSubClassOfAxiom newR2CI = OWLManager.getOWLDataFactory().getOWLSubClassOfAxiom(newExRC, newSpurious);
                    setOfNewCIs.add(newR2CI);
                    // (r3)
                    OWLObjectPropertyExpression role = exRC.getProperty();
                    if (!(role instanceof OWLObjectProperty)) {
                        throw new CQException("[" + TqCQRewriter.class.getSimpleName() + "] Found " + role + ", a non-simple role!");
                    }
                    OWLObjectProperty simpleRole = (OWLObjectProperty) role;
                    for (Atom at : q0.getSetOfAtoms()) {
                        if (at instanceof RoleAtom) {
                            RoleAtom rat = (RoleAtom) at;
                            if (!rat.getXVar().equals(x)) {
                                continue;
                            }
                            if (!rat.getRole().equals(simpleRole)) {
                                continue;
                            }
                            Variable y = rat.getYVar();
                            Set<OWLClassExpression> tlcsOfC = exRC.getFiller().asConjunctSet();
                            Set<OWLClassExpression> r3LhsTlcs = Annotations.lhsAnnotation(tlcsOfC, y);
                            OWLClassExpression r3Lhs = OntologyUtils.getClsOrIntersection(r3LhsTlcs);
                            OWLSubClassOfAxiom newR3CI = OWLManager.getOWLDataFactory().getOWLSubClassOfAxiom(r3Lhs, newSpurious);
                            setOfNewCIs.add(newR3CI);
                        }
                    }
                } // for exRC
            } // for x
        } // for CIs
        // Construct A^x...A^z => N:
        OWLClass clsN = CQContext.getINSTANCE().getClsN();
        Set<OWLClassExpression> setOfAnnotatedClasses = new HashSet<>();
        for (Atom at : q0.getSetOfAtoms()) {
            if (at instanceof ClassAtom) {
                ClassAtom classAtom = (ClassAtom) at;
                OWLClass cls = classAtom.getCls();
                Variable x = classAtom.getVar();
                OWLClass annotatedCls = CQContext.getINSTANCE().annotateCls(cls, x);
                setOfAnnotatedClasses.add(annotatedCls);
            }
        }
        OWLClassExpression allAnnClasses = OntologyUtils.getClsOrIntersection(setOfAnnotatedClasses);
        OWLSubClassOfAxiom lastCI = OWLManager.getOWLDataFactory().getOWLSubClassOfAxiom(allAnnClasses, clsN);
        setOfNewCIs.add(lastCI);
        return setOfNewCIs;
    }

}
