/*
 * Copyright 2014, 2015 Peter Hansen and İnanç Seylan,
 *           2016, 2017 Peter Hansen.
 * 
 * This file is part of Grind v2.
 * 
 * Grind is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Grind is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Grind.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.unibremen.informatik.tdki.grind2.cq;

import de.unibremen.informatik.tdki.grind2.context.CQContext;
import de.unibremen.informatik.tdki.grind2.cq.components.Variable;
import de.unibremen.informatik.tdki.grind2.utilities.Signature;
import de.unibremen.informatik.tdki.grind2.cq.components.RoleAtom;
import de.unibremen.informatik.tdki.grind2.cq.components.ClassAtom;
import de.unibremen.informatik.tdki.grind2.cq.components.Atom;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Random;
import java.util.Set;
import org.semanticweb.owlapi.apibinding.OWLManager;
import org.semanticweb.owlapi.model.OWLAxiom;
import org.semanticweb.owlapi.model.OWLClass;
import org.semanticweb.owlapi.model.OWLClassExpression;
import org.semanticweb.owlapi.model.OWLObjectProperty;
import org.semanticweb.owlapi.model.OWLSubClassOfAxiom;

/**
 *
 * @author hansen
 */
public class AvarCQ extends CQ {

    private Set<OWLAxiom> newAxioms;

    public Set<OWLAxiom> getNewAxioms() {
        return newAxioms;
    }

    public AvarCQ(Set<Variable> avars, Set<Atom> setOfAtoms) throws CQException {
        super(avars, setOfAtoms);

        // Check we have at least one avar:
        if (avars.isEmpty()) {
            throw new CQException("[" + this.getClass().getSimpleName() + "] Tried to construct AvarCQ without avars!");
        }

        // Check all used vars are avars:
        Set<Variable> setOfAllVars = new HashSet<>();
        for (Atom a : setOfAtoms) {
            setOfAllVars.addAll(a.getVars());
        }
        if (!setOfAllVars.equals(avars)) {
            System.out.println(setOfAllVars + " != " + avars);
            throw new CQException("[" + this.getClass().getSimpleName() + "] Tried to construct AvarCQ with qvars!");
        }
    }


    /**
     * Returns this AvarCQ as AvarCQ with 0 or 1 (potentially fresh) CN at a
     * variable. Also returns the new CIs C => A_C that need to be included in
     * T.
     *
     * @return
     * @throws CQException
     */
    @Deprecated
    public Set<OWLClass> getCNsFromAvarOptimization() throws CQException {

        Map<Variable, Set<OWLClassExpression>> x2setOfClsMap = new HashMap<>();
        for (Atom at : this.getSetOfAtoms()) {
            if (at instanceof ClassAtom) {
                ClassAtom cat = (ClassAtom) at;
                Variable x = cat.getVar();
                if (x2setOfClsMap.containsKey(x)) {
                    Set<OWLClassExpression> setOfCls = x2setOfClsMap.get(x);
                    setOfCls.add(cat.getCls());
                } else {
                    Set<OWLClassExpression> setOfCls = new HashSet<>();
                    setOfCls.add(cat.getCls());
                    x2setOfClsMap.put(x, setOfCls);
                }
            }
        }

        Set<OWLClass> setOfCNsToTest = new HashSet<>();
        this.newAxioms = new HashSet<>();
        for (Variable x : x2setOfClsMap.keySet()) {
            OWLClass clsAtX;
            Set<OWLClassExpression> classesAtX = x2setOfClsMap.get(x);
            if (classesAtX.size() == 1) {
                clsAtX = (OWLClass) classesAtX.iterator().next();
                setOfCNsToTest.add(clsAtX);
            } else {
                OWLClassExpression conceptC = OWLManager.getOWLDataFactory().getOWLObjectIntersectionOf(classesAtX);
                clsAtX = OWLManager.getOWLDataFactory().getOWLClass("A_C"
                        + CQContext.getINSTANCE().getCounter(), CQContext.getINSTANCE().getPm());
                setOfCNsToTest.add(clsAtX);
                OWLSubClassOfAxiom newCI = OWLManager.getOWLDataFactory().getOWLSubClassOfAxiom(conceptC, clsAtX);
                this.newAxioms.add(newCI);
            }
        }
        return setOfCNsToTest;
    }


    public static AvarCQ getRandomAvarCQ(int numberOfVars, Signature sigma) throws CQException {
//        if (sigma.getSetOfClasses().isEmpty()) {
//            throw new CQException("[" + AvarCQ.class.getSimpleName() + "] Tried to construct AvarCQ with empty class signature!");
//        }
        if (sigma.getSetOfRoles().isEmpty()) {
            throw new CQException("[" + AvarCQ.class.getSimpleName() + "] Tried to construct AvarCQ with empty role signature!");
        }
        if (numberOfVars == 0) {
            throw new CQException("[" + AvarCQ.class.getSimpleName() + "] Tried to construct AvarCQ with 0 variables!");
        }
        // Create all vars:
        List<Variable> listOfAvars = new ArrayList<>();
        for (int i = 0; i < numberOfVars; i++) {
            listOfAvars.add(new Variable());
        }

        // Put classes and roles into lists:
        List<OWLClass> listOfClasses = new ArrayList<>();
        listOfClasses.addAll(sigma.getSetOfClasses());
        List<OWLObjectProperty> listOfRoles = new ArrayList<>();
        listOfRoles.addAll(sigma.getSetOfRoles());

        // Incidence matrix for classes and roles:
        int numberOfClasses = listOfClasses.size();
        int numberOfRoles = listOfRoles.size();
        int sizeOfCRArray = numberOfClasses + numberOfRoles * numberOfVars;
        boolean[][] classAndRoleArray = new boolean[sizeOfCRArray][numberOfVars];

        // Hold truth values for "variable already used". Defaults to false.
        boolean[] varUsed = new boolean[numberOfVars];

        assureAtLeastOneOccurrence(numberOfVars, varUsed, sizeOfCRArray,
                numberOfClasses, classAndRoleArray, numberOfRoles);

        fillMatrixRandomlyZeroToTwoCNs(sizeOfCRArray, numberOfVars,
                classAndRoleArray, numberOfClasses); //, CQ.getATOMPROB());
        //printMatrix(sizeOfCRArray, numberOfVars, classAndRoleArray);

        Set<Atom> setOfAtoms = new HashSet<>();
        // Fill class atoms:
        for (int line = 0; line < numberOfClasses; line++) {
            for (int varNr = 0; varNr < numberOfVars; varNr++) {
                if (classAndRoleArray[line][varNr]) {
                    Atom at = new ClassAtom(listOfClasses.get(line), listOfAvars.get(varNr));
                    setOfAtoms.add(at);
                }
            }
        }
        // Fill role atoms:
        for (int line = numberOfClasses; line < sizeOfCRArray; line++) {
            for (int varNr = 0; varNr < numberOfVars; varNr++) {
                if (classAndRoleArray[line][varNr]) {
                    Atom at = new RoleAtom(
                            listOfRoles.get((line - numberOfClasses) / numberOfVars),
                            listOfAvars.get(varNr), listOfAvars.get((line - numberOfClasses) % numberOfVars)
                    );
                    setOfAtoms.add(at);
                }
            }
        }
//        System.out.println("listOfAvars " + listOfAvars);
//        System.out.println("setOfAtoms " + setOfAtoms);
        AvarCQ avarCQ = new AvarCQ(new HashSet<>(listOfAvars), setOfAtoms);
        return avarCQ;
    }

    private static void assureAtLeastOneOccurrence(int numberOfVars, boolean[] varUsed,
            int sizeOfCRArray, int numberOfClasses, boolean[][] classAndRoleArray, int numberOfRoles) {

        // Distribute one occurrence of each variable:
        for (int i = 0; i < numberOfVars; i++) {
            if (!varUsed[i]) {
                // Find a place where to use x_i:
                Random rand = new Random();
                int whereToUse = rand.nextInt(sizeOfCRArray - numberOfClasses) + numberOfClasses;

//                // Ok, is it a class atom?
//                if (whereToUse < numberOfClasses) {
//                    classAndRoleArray[whereToUse][i] = true;
//                    varUsed[i] = true;
//                } else // it is a role atom
//                {
                classAndRoleArray[whereToUse][i] = true;
                // which other variable is used now?
                varUsed[(whereToUse - numberOfClasses) % numberOfVars] = true;
//                }
            }
        }
    }

    private static void fillMatrixRandomly(int sizeOfCRArray, int numberOfVars, boolean[][] classAndRoleArray, double prob) {
        // Now fill the rest of the matrix randomly:
        for (int line = 0; line < sizeOfCRArray; line++) {
            for (int varNr = 0; varNr < numberOfVars; varNr++) {
                if (Math.random() < prob) {
                    classAndRoleArray[line][varNr] = true;
                }
            }
        }
    }

    private static void fillMatrixRandomlyZeroToTwoCNs(int sizeOfCRArray,
            int numberOfVars, boolean[][] classAndRoleArray, int numberOfClasses) {
        // Now fill the rest of the matrix randomly:
        Random rand = new Random();

        // Every variable...
        for (int varNr = 0; varNr < numberOfVars; varNr++) {
            // gets 0, 1 or 2 concept names:
            for (int numberOfCNsHere = 0; numberOfCNsHere < rand.nextInt(3); numberOfCNsHere++) {
                classAndRoleArray[rand.nextInt(numberOfClasses)][varNr] = true;
            }
        }
    }

    private static void printMatrix(int sizeOfCRArray, int numberOfVars, boolean[][] classAndRoleArray) {
        for (int line = 0; line < sizeOfCRArray; line++) {
            for (int varNr = 0; varNr < numberOfVars; varNr++) {
                if (classAndRoleArray[line][varNr]) {
                    System.out.print("1");
                } else {
                    System.out.print("0");
                }
            }
            System.out.println();
        }
    }


}
