/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package de.unibremen.informatik.tdki.grind2.cq;

import com.google.common.collect.Sets;
import de.unibremen.informatik.tdki.grind.context.OriginalOntology;
import de.unibremen.informatik.tdki.grind2.context.CQContext;
import de.unibremen.informatik.tdki.grind2.cq.components.Atom;
import de.unibremen.informatik.tdki.grind2.cq.components.RoleAtom;
import de.unibremen.informatik.tdki.grind2.cq.components.Variable;
import de.unibremen.informatik.tdki.grind2.utilities.ConceptTreeNode;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import org.semanticweb.owlapi.apibinding.OWLManager;
import org.semanticweb.owlapi.model.OWLAxiom;
import org.semanticweb.owlapi.model.OWLClass;
import org.semanticweb.owlapi.model.OWLClassExpression;
import org.semanticweb.owlapi.model.OWLObjectPropertyExpression;
import org.semanticweb.owlapi.model.OWLOntology;
import org.semanticweb.owlapi.model.OWLOntologyCreationException;
import org.semanticweb.owlapi.model.OWLSubClassOfAxiom;

/**
 *
 * @author hansen
 */
public class ConCQ extends CQ {
    
    private Variable root;
    private OWLClass freshConceptname;

    /**
     * We can put a non-tree-shaped set of atoms in here. getConcept(xRoot) will
     * get the concept rooted at xRoot!
     *
     * @param setOfAtoms
     * @param root
     * @throws CQException
     */
    public ConCQ(Set<Atom> setOfAtoms, Variable root) throws CQException {
        super(Sets.newHashSet(root), setOfAtoms);
        this.root = root;
        if (!this.getVars().contains(root)) {
            throw new CQException("[" + this.getClass().getSimpleName()
                    + "] Tried to construct TreeCQ with root not in vars!");
        }
        if (!this.isTreeShaped()) {
            throw new CQException("[" + this.getClass().getSimpleName()
                    + "] Tried to construct non-treeshaped TreeCQ!");
        }
        this.freshConceptname = OWLManager.getOWLDataFactory().getOWLClass(
                "A_C" + CQContext.getINSTANCE().getCounter(),
                CQContext.getINSTANCE().getPm());
    }

    public OWLClass getFreshConceptname() {
        return this.freshConceptname;
    }


   public final boolean isTreeShaped() {
        // root has to have indegree of zero:
        if (this.indegree(root) != 0) {
            return false;
        }
        // Trees have to be connected:
        if (!this.isConnected()) {
            return false;
        }
        // Check that we don't have multiple edges r(x,y), s(x,y):
        Set<List<Variable>> setOfTuples = new HashSet<>();
        for (Atom at : this.getSetOfAtoms()) {
            if (at instanceof RoleAtom) {
                List<Variable> tuple = new ArrayList<>();
                tuple.add(((RoleAtom) at).getXVar());
                tuple.add(((RoleAtom) at).getYVar());
                if (setOfTuples.contains(tuple)) {
                    return false;
                } else {
                    setOfTuples.add(tuple);
                }
            }
        }
        // And they have to be acyclic:
        if (!this.isAcyclic()) {
            return false;
        }
        return true;
    }

    /**
     * Return the OWLClassExpression that is associated with a TreeCQ.
     *
     * @return
     * @throws de.unibremen.informatik.tdki.grind2.cq.CQException
     */
    public OWLClassExpression getConcept() throws CQException {
        if (this.getSetOfAtoms().isEmpty()) {
            return OWLManager.createOWLOntologyManager().getOWLDataFactory().getOWLThing();
        }
        OWLClassExpression concept;
        // all visited Variables:
        Map<Variable, ConceptTreeNode> vars2TreeNodeMap = new HashMap<>();
        fillTreeNodes(vars2TreeNodeMap);
        return visitConcept(vars2TreeNodeMap.get(root));
    }

    private OWLClassExpression visitConcept(ConceptTreeNode xNode) {
        if (xNode.isLeaf()) {
            // Leaf node!
            Set<OWLClass> setOfClasses = xNode.getSetOfClasses();
            switch (setOfClasses.size()) {
                case 0:
                    // No classes! Return Top:
                    return OWLManager.getOWLDataFactory().getOWLThing();
                case 1:
                    // One class. Don't do Intersection:
                    for (OWLClass cls : setOfClasses) {
                        return cls;
                    }
                default:
                    // Has some classes, return an Intersection:
                    return OWLManager.getOWLDataFactory().getOWLObjectIntersectionOf(setOfClasses);
            }
        } else {
            // Non-leaf node!
            Set<OWLClassExpression> setOfClassEx = new HashSet<>();
            for (ConceptTreeNode child : xNode.getChildren()) {
                OWLClassExpression childClass = visitConcept(child);
                OWLObjectPropertyExpression role = child.getReachingRole();
                setOfClassEx.add(OWLManager.getOWLDataFactory().getOWLObjectSomeValuesFrom(role, childClass));
            }
            // also add all classes
            setOfClassEx.addAll(xNode.getSetOfClasses());

            if (setOfClassEx.size() == 1) {
                // One ClassExpr. Don't do Intersection:
                for (OWLClassExpression cls : setOfClassEx) {
                    return cls;
                }
            }
            // Several ones. Do Intersection:
            return OWLManager.getOWLDataFactory().getOWLObjectIntersectionOf(setOfClassEx);

        }
    }

}
