/*
 * Copyright 2014, 2015 Peter Hansen and İnanç Seylan,
 *           2016, 2017 Peter Hansen.
 * 
 * This file is part of Grind v2.
 * 
 * Grind is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Grind is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Grind.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.unibremen.informatik.tdki.grind2.cq;

import de.unibremen.informatik.tdki.grind2.cq.components.Atom;
import de.unibremen.informatik.tdki.grind2.cq.components.Variable;
import java.util.Objects;
import java.util.Set;
import org.semanticweb.owlapi.model.OWLObjectProperty;

/**
 *
 * @author hansen
 */
public class Fork {

    private final OWLObjectProperty role;
    private final Set<Variable> sourceVars;
    private final Variable targetVar;

    public Fork(OWLObjectProperty role, Set<Variable> sourceVars, Variable targetVar) {
        this.role = role;
        this.sourceVars = sourceVars;
        this.targetVar = targetVar;
    }

    public Set<Atom> getIdentification() {
        //
        return null;
    }

    public OWLObjectProperty getRole() {
        return role;
    }

    public Set<Variable> getSourceVars() {
        return sourceVars;
    }

    public Variable getTargetVar() {
        return targetVar;
    }

    @Override
    public String toString() {
        return "Fork{" + "role=" + role + ", sourceVars=" + sourceVars + ", targetVar=" + targetVar + '}';
    }

    @Override
    public int hashCode() {
        int hash = 7;
        hash = 97 * hash + Objects.hashCode(this.role);
        hash = 97 * hash + Objects.hashCode(this.sourceVars);
        hash = 97 * hash + Objects.hashCode(this.targetVar);
        return hash;
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj) {
            return true;
        }
        if (obj == null) {
            return false;
        }
        if (getClass() != obj.getClass()) {
            return false;
        }
        final Fork other = (Fork) obj;
        if (!Objects.equals(this.role, other.role)) {
            return false;
        }
        if (!Objects.equals(this.sourceVars, other.sourceVars)) {
            return false;
        }
        if (!Objects.equals(this.targetVar, other.targetVar)) {
            return false;
        }
        return true;
    }

}
