/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package de.unibremen.informatik.tdki.grind2.cq;

import com.google.common.collect.Sets;
import de.unibremen.informatik.tdki.grind2.context.CQContext;
import de.unibremen.informatik.tdki.grind2.cq.components.Atom;
import de.unibremen.informatik.tdki.grind2.cq.components.ClassAtom;
import de.unibremen.informatik.tdki.grind2.cq.components.RoleAtom;
import de.unibremen.informatik.tdki.grind2.cq.components.Variable;
import de.unibremen.informatik.tdki.grind2.datalog.components.DatalogRoleAtom;
import de.unibremen.informatik.tdki.grind2.datalog.components.DatalogVariable;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;
import org.semanticweb.owlapi.apibinding.OWLManager;
import org.semanticweb.owlapi.model.OWLAxiom;
import org.semanticweb.owlapi.model.OWLClass;
import org.semanticweb.owlapi.model.OWLClassExpression;
import org.semanticweb.owlapi.model.OWLObjectProperty;
import org.semanticweb.owlapi.model.OWLSubClassOfAxiom;

/**
 *
 * @author hansen
 */
public class TqCQ extends CQ {

    private final CQ core;
    private final Set<Variable> coreVars;
    private final Map<Variable, OWLClassExpression> root2treeMap;
    private final Map<Variable, OWLClass> root2ACmap;
    private Set<OWLAxiom> newAxioms;

    public TqCQ(Set<Variable> avars, Set<Atom> setOfAtoms) throws CQException {
        super(avars, setOfAtoms);

        this.core = ((CQ) this).getCore();
        this.coreVars = this.core.getVars();
        for (Variable x : this.coreVars) {
            if (!x.containsSomeAvarOf(avars)) {
                throw new CQException("[" + this.getClass().getSimpleName() + "] Tried to construct TqCQ with qvars in core!");
            }
        }
        // Should also check here that existential parts are tree-shaped!

        this.root2treeMap = getMapRoots2Trees();
        this.root2ACmap = new HashMap<>();
    }

    private Map<Variable, OWLClassExpression> getMapRoots2Trees() throws CQException {
        Map<Variable, OWLClassExpression> resultMap = new HashMap<>();

        // Have to generate a copy of the query without core ROLE atoms, so that
        // we can correctly extract the trees:
        Set<Atom> emptyCoreAtomSet = new HashSet<>(this.getSetOfAtoms());

        for (Atom at : this.core.getSetOfAtoms()) {
            if (at instanceof RoleAtom) {
                emptyCoreAtomSet.remove(at);
            }
        }
        CQ emptyCoreCopy = new CQ(this.avars, emptyCoreAtomSet);

        Set<Variable> roots = Sets.union(this.coreVars, this.avars);
        for (Variable x : roots) {
            TreeCQ treeCQAtX = emptyCoreCopy.getTreeCQInAcyclicPartAt(x);
            OWLClassExpression conceptAtX = treeCQAtX.getConcept();
            resultMap.put(x, conceptAtX);
        }
        return resultMap;
    }

    /**
     * Returns this TqCQ as AvarCQ: only the root, and fresh concept names for
     * the existential trees / A^B at some x. Also returns the new CIs C => A_C
     * that need to be included in T. Also returns the Map x -> A_Cx.
     *
     * @return
     * @throws CQException
     */
    public AvarCQ getReducedAvarCQ() throws CQException {
        Map<Variable, OWLClass> resultMap = new HashMap<>();

        this.newAxioms = new HashSet<>();

        Set<Atom> newSetOfAtoms = new HashSet<>();
        for (Atom at : this.core.getSetOfAtoms()) {
            if (at instanceof RoleAtom) {
                newSetOfAtoms.add(at);
            }
        }

        Set<Variable> roots = this.root2treeMap.keySet();
        for (Variable x : roots) {
            OWLClassExpression conceptC = this.root2treeMap.get(x);
            if (CQContext.getINSTANCE().isVerbose()) {
                System.out.println("Variable " + x + ", concept: " + conceptC);
            }
            if (conceptC.equals(OWLManager.getOWLDataFactory().getOWLThing())) {
                continue;
            }

            OWLClass clsAtX;

            // (1) need to create CI?            
            if (conceptC instanceof OWLClass) {
                clsAtX = (OWLClass) conceptC;
            } else {
                clsAtX = OWLManager.getOWLDataFactory()
                        .getOWLClass("A_C" + x + CQContext.getINSTANCE().getCounter(), CQContext.getINSTANCE().getPm());
                OWLSubClassOfAxiom newCI = OWLManager.getOWLDataFactory().getOWLSubClassOfAxiom(conceptC, clsAtX);
                this.newAxioms.add(newCI);
            }

            // (2) add atom A_Cx(x) to newSetOfAtoms:
            Atom newAtom = new ClassAtom(clsAtX, x);
            newSetOfAtoms.add(newAtom);
            this.root2ACmap.put(x, clsAtX);
        }

        //AvarCQ avarCQ = new AvarCQ(this.avars, newSetOfAtoms);
        AvarCQ avarCQ = new AvarCQ(roots, newSetOfAtoms);
        return avarCQ;
    }

    public Map<Variable, OWLClass> getRoot2ACmap() {
        return root2ACmap;
    }

    public Set<OWLAxiom> getNewAxioms() {
        return newAxioms;
    }

    public Set<DatalogRoleAtom> getCoreDatalogRoleAtoms() throws CQException {
        Set<DatalogRoleAtom> resultSet = new HashSet<>();

        // Find all q0 core role atoms, put them into body atoms:
        for (Atom at : this.getCore().getSetOfAtoms()) {
            if (at instanceof RoleAtom) {
                RoleAtom rat = (RoleAtom) at;
                OWLObjectProperty role = rat.getRole();
                Variable x = rat.getXVar();
                Variable y = rat.getYVar();

                int indexX = x.getIndices().hashCode();
                int indexY = y.getIndices().hashCode();

                DatalogVariable xDVar = new DatalogVariable(indexX);
                DatalogVariable yDVar = new DatalogVariable(indexY);
                DatalogRoleAtom dRat = new DatalogRoleAtom(role, xDVar, yDVar);

                resultSet.add(dRat);
            }
        }
        return resultSet;
    }

    public Set<DatalogVariable> getCoreDatalogVariables() throws CQException {
        Set<DatalogVariable> dVars = new HashSet<>();

        for (Variable x : this.getCore().getVars()) {
            int indexX = x.getIndices().hashCode();
            DatalogVariable xDVar = new DatalogVariable(indexX);
            dVars.add(xDVar);
        }
        // Special case: core empty:
        if (this.getCore().getVars().isEmpty()) {
            for (Variable x : this.getVars()) {
                if (x.containsSomeAvarOf(this.getAvars())) {
                    int indexX = x.getIndices().hashCode();
                    dVars.add(new DatalogVariable(indexX));
                }
            }
        }
        return dVars;
    }

}
