/*
 * Copyright 2014, 2015 Peter Hansen and İnanç Seylan,
 *           2016, 2017 Peter Hansen.
 * 
 * This file is part of Grind v2.
 * 
 * Grind is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Grind is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Grind.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.unibremen.informatik.tdki.grind2.cq.components;

import java.util.Collections;
import java.util.HashSet;
import java.util.Objects;
import java.util.Set;
import org.semanticweb.owlapi.model.OWLClass;

/**
 *
 * @author hansen
 */
public class ClassAtom extends Atom {

    private  final OWLClass cls;
    private  final Variable x;

    public ClassAtom(OWLClass cls, Variable x) {
        this.cls = cls;
        this.x = x;
    }

    public OWLClass getCls() {
        return cls;
    }

    public Variable getVar() {
        return x;
    }

    @Override
    public Set<Variable> getVars() {
        Set<Variable> vars = new HashSet<>();
        if (x != null) {
            vars.add(x);
        }
        return vars;
    }

    @Override
    public String toString() {
        return cls + "(" + x + ')';
    }

    @Override
    public int hashCode() {
        int hash = 7;
        hash = 11 * hash + Objects.hashCode(this.cls);
        hash = 11 * hash + Objects.hashCode(this.x);
        return hash;
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj) {
            return true;
        }
        if (obj == null) {
            return false;
        }
        if (getClass() != obj.getClass()) {
            return false;
        }
        final ClassAtom other = (ClassAtom) obj;
        if (!Objects.equals(this.cls, other.cls)) {
            return false;
        }
        if (!Objects.equals(this.x, other.x)) {
            return false;
        }
        return true;
    }

    @Override
    public Atom replaceIdentifiedVars(Variable newSourceVar) {

        if (!Collections.disjoint(this.x.getIndices(), newSourceVar.getIndices())) {
            Variable newX = newSourceVar;
            return new ClassAtom(this.cls, newX);
        }
        return this;

    }

}
