/*
 * Copyright 2014, 2015 Peter Hansen and İnanç Seylan,
 *           2016, 2017 Peter Hansen.
 * 
 * This file is part of Grind v2.
 * 
 * Grind is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Grind is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Grind.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.unibremen.informatik.tdki.grind2.cq.components;

import java.util.Collections;
import java.util.HashSet;
import java.util.Objects;
import java.util.Set;
import org.semanticweb.owlapi.model.OWLObjectProperty;

/**
 *
 * @author hansen
 */
public class RoleAtom extends Atom {

    private final OWLObjectProperty role;
    private final Variable x;
    private final Variable y;

    public RoleAtom(OWLObjectProperty role, Variable x, Variable y) {
        this.role = role;
        this.x = x;
        this.y = y;
    }

    public Variable getXVar() {
        return x;
    }

    public Variable getYVar() {
        return y;
    }

    public OWLObjectProperty getRole() {
        return role;
    }

    @Override
    public Set<Variable> getVars() {
        Set<Variable> vars = new HashSet<>();
        if (x != null) {
            vars.add(x);
        }
        if (y != null) {
            vars.add(y);
        }
        return vars;
    }

    @Override
    public String toString() {
        return role + "(" + x + ", " + y + ')';
    }

    @Override
    public int hashCode() {
        int hash = 3;
        hash = 61 * hash + Objects.hashCode(this.role);
        hash = 61 * hash + Objects.hashCode(this.x);
        hash = 61 * hash + Objects.hashCode(this.y);
        return hash;
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj) {
            return true;
        }
        if (obj == null) {
            return false;
        }
        if (getClass() != obj.getClass()) {
            return false;
        }
        final RoleAtom other = (RoleAtom) obj;
        if (!Objects.equals(this.role, other.role)) {
            return false;
        }
        if (!Objects.equals(this.x, other.x)) {
            return false;
        }
        if (!Objects.equals(this.y, other.y)) {
            return false;
        }
        return true;
    }

    @Override
    public Atom replaceIdentifiedVars(Variable newSourceVar) {

        Variable newX = this.x;
        Variable newY = this.y;
        boolean changed = false;

        if (!Collections.disjoint(this.x.getIndices(), newSourceVar.getIndices())) {
            newX = newSourceVar;
            changed = true;
        }
        if (!Collections.disjoint(this.y.getIndices(), newSourceVar.getIndices())) {
            newY = newSourceVar;
            changed = true;
        }
        if (changed) {
            return new RoleAtom(this.role, newX, newY);
        } else {
            return this;
        }
    }

}
