/*
 * Copyright 2014, 2015 Peter Hansen and İnanç Seylan,
 *           2016, 2017 Peter Hansen.
 * 
 * This file is part of Grind v2.
 * 
 * Grind is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Grind is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Grind.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.unibremen.informatik.tdki.grind2.cq.components;

import com.google.common.collect.Sets;
import de.unibremen.informatik.tdki.grind2.cq.CQException;
import java.util.HashSet;
import java.util.Objects;
import java.util.Set;

/**
 *
 * @author hansen
 */
public class Variable implements Comparable<Variable> {

    static int indexCounter = 0;

    Set<Integer> indices;

    public Variable() {
        indices = new HashSet<>();
        indices.add(indexCounter);
        indexCounter++;
    }

    public Variable(int index) {
        indices = Sets.newHashSet(index);
    }

    public Variable(Set<Variable> setOfVars) {
        Set<Integer> unionIndexSet = new HashSet<>();
        for (Variable x : setOfVars) {
            unionIndexSet.addAll(x.getIndices());
        }
        indices = unionIndexSet;
    }

    public Set<Integer> getIndices() {
        return indices;
    }

    public int getSingleIndex() throws CQException {
        if (indices.size() != 1) {
            throw new CQException("[" + this.getClass().getSimpleName()
                    + "] Asked singleIndex from non-simple variable (index set size != 1)!");
        }
        return indices.iterator().next();
    }

    public Set<Integer> getAvarIndices(Set<Integer> avarIdxset) {
        return Sets.intersection(indices, avarIdxset);
    }

    public boolean containsSomeAvarOf(Set<Variable> avars) {
        Set<Integer> avarIndexSet = new HashSet<>();
        for (Variable x : avars) {
            avarIndexSet.addAll(x.getIndices());
        }
        return !Sets.intersection(avarIndexSet, indices).isEmpty();

    }

    @Override
    public int compareTo(Variable t) {
        return this.toString().compareTo(t.toString());
    }

    @Override
    public int hashCode() {
        int hash = 5;
        hash = 13 * hash + Objects.hashCode(this.indices);
        return hash;
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj) {
            return true;
        }
        if (obj == null) {
            return false;
        }
        if (getClass() != obj.getClass()) {
            return false;
        }
        final Variable other = (Variable) obj;
        if (!Objects.equals(this.indices, other.indices)) {
            return false;
        }
        return true;
    }

    @Override
    public String toString() {
        return "x" + indices;
    }

}
