/*
 * Copyright 2014, 2015 Peter Hansen and İnanç Seylan,
 *           2016, 2017 Peter Hansen.
 * 
 * This file is part of Grind v2.
 * 
 * Grind is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Grind is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Grind.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.unibremen.informatik.tdki.grind2.runnables;


public class RunHandcodedCQs {

    /*
    
//  Runtimes of toy query (using other CNs) on these ontologies:
//
//     *  envo.owl      //   11s
//     *  fbbi.owl      //    4s
//     *  mohse.owl     // 1m16s
//        nbo.owl       //   15s
//     *  not-galen.owl // 3m41s
//     *  so.owl        //   39s
//        XP.owl        //   41s
//
    public static void main(String[] args) throws OWLOntologyCreationException, CQException, Exception {
        long t0 = System.currentTimeMillis();

        CQContext.load("./grind-test-ontologies/envo.owl");
        System.out.println("### Query envo01");
        runENVO_wasteTreatmentPlant();
        System.out.println("### Query envo02");
        runENVO_brewery();
        System.out.println("### Query envo03");
        runENVO_glacier();
        System.out.println("### Query envo04");
        runENVO_nuclearPowerPlant();
        System.out.println("### Query envo05");
        runENVO_river();
        System.out.println("### Query envo06");
        runENVO_iceberg();
        System.out.println("### Query envo07");
        runENVO_hospital();
        System.out.println("### Query envo08");
        runENVO_mountainRange();
        System.out.println("### Query envo09");
        runENVO_volcanicCrater();
        System.out.println("### Query envo10");
        runENVO_karst();
//
        CQContext.load("./grind-test-ontologies/fbbi.owl");
        System.out.println("### Query fbbi01");
        runFBbi_negativeStaining();
        System.out.println("### Query fbbi02");
        runFBbi_eyfp();
        System.out.println("### Query fbbi03");
        runFBbi_luciferYellow();
        System.out.println("### Query fbbi04");
        runFBbi_cryostatSectionedTissue();
        System.out.println("### Query fbbi05");
        runFBbi_areaDetector();
        System.out.println("### Query fbbi06");
        runFBbi_detectionOfPhotons();
        System.out.println("### Query fbbi07");
        runFBbi_coagulationFixativeFixedTissue();
        System.out.println("### Query fbbi08");
        runFBbi_opticalHighlighter();
        System.out.println("### Query fbbi09");
        runFBbi_greyScaleGraphic();
        System.out.println("### Query fbbi10");
        runFBbi_arrayScanConfocal();
//
        CQContext.load("./grind-test-ontologies/mohse.owl");
        System.out.println("### Query mohse01");
        runMOHSE_endocarditis();
        System.out.println("### Query mohse02");
        runMOHSE_tissue();
        System.out.println("### Query mohse03");
        runMOHSE_alcohol();
        System.out.println("### Query mohse04");
        runMOHSE_giardia();
        System.out.println("### Query mohse05");
        runMOHSE_duct();
        System.out.println("### Query mohse06");
        runMOHSE_voluntaryMovement();
        System.out.println("### Query mohse07");
        runMOHSE_tetracycline();
        System.out.println("### Query mohse08");
        runMOHSE_teichoicAcid();
        System.out.println("### Query mohse09");
        runMOHSE_bordatella();
        System.out.println("### Query mohse10");
        runMOHSE_brain();
//
        CQContext.load("./grind-test-ontologies/not-galen.owl");
        System.out.println("### Query ng01");
        runNotGalen_endocarditis();
        System.out.println("### Query ng02");
        runNotGalen_tissue();
        System.out.println("### Query ng03");
        runNotGalen_alcohol();
        System.out.println("### Query ng04");
        runNotGalen_giardia();
        System.out.println("### Query ng05");
        runNotGalen_duct();
        System.out.println("### Query ng06");
        runNotGalen_voluntaryMovement();
        System.out.println("### Query ng07");
        runNotGalen_tetracycline();
        System.out.println("### Query ng08");
        runNotGalen_teichoicAcid();
        System.out.println("### Query ng09"); // timeout
        //runNotGalen_bordatella();
        System.out.println("### Query ng10");  // memout
        //runNotGalen_brain();
//
        CQContext.load("./grind-test-ontologies/so.owl");
        System.out.println("### Query so01");
        runSO_chromosomeArm();
        System.out.println("### Query so02");
        runSO_pipBox();
        System.out.println("### Query so03");
        runSO_proviralGene();
        System.out.println("### Query so04");
        runSO_mobileElementInsertion();
        System.out.println("### Query so05");
        runSO_spliceJunction();
        System.out.println("### Query so06");
        runSO_engineeredFusionGene();
        System.out.println("### Query so07");
        runSO_insertionBreakpoint();
        System.out.println("### Query so08");
        runSO_epitope();
        System.out.println("### Query so09");
        runSO_biologicalRegion();
        System.out.println("### Query so10"); // memout
        //runSO_alanine();
//        
        System.out.println("### ");
        System.out.println("### Computation of all queries took " + (System.currentTimeMillis() - t0) + " ms.");
    }




    public static void runENVO_hospital() throws CQException, Exception {
        Signature sigma = CQContext.getINSTANCE().getOriginalSignature();

        Variable x0 = new Variable(0);
        Variable x1 = new Variable(1);
        Variable x2 = new Variable(2);

        OWLClass hospital = sigma.getCls("owlENVO_00002173");
        OWLClass nuclearWeaponsTestSite = sigma.getCls("owlENVO_00000560");
        OWLClass desert = sigma.getCls("owlENVO_00000097");
        OWLObjectProperty locatedIn = sigma.getRole("owlENVO_located_in");

        Set<Atom> setOfAtoms = new HashSet<>();
        setOfAtoms.add(new ClassAtom(hospital, x0));
        setOfAtoms.add(new ClassAtom(nuclearWeaponsTestSite, x1));
        setOfAtoms.add(new ClassAtom(desert, x2));
        setOfAtoms.add(new RoleAtom(locatedIn, x0, x2));
        setOfAtoms.add(new RoleAtom(locatedIn, x1, x2));
        Set<Variable> avarSet = Sets.newHashSet(x0);
        RootedCQ q0 = new RootedCQ(avarSet, setOfAtoms);

        QueryCaller.runRootedCQ(q0, Thread.currentThread().getStackTrace()[1].getMethodName(), "envo07");
    }

    private static void runENVO_wasteTreatmentPlant() throws OWLOntologyCreationException, CQException, Exception {
        Signature sigma = CQContext.getINSTANCE().getOriginalSignature();

        Variable x0 = new Variable(0);
        Variable x1 = new Variable(1);

        OWLClass wasteTreatmentPlant = sigma.getCls("owlENVO_00002272");
        OWLClass protectedArea = sigma.getCls("owlENVO_00000358");
        OWLObjectProperty locatedIn = sigma.getRole("owlENVO_located_in");

        Set<Atom> setOfAtoms = new HashSet<>();
        setOfAtoms.add(new ClassAtom(wasteTreatmentPlant, x0));
        setOfAtoms.add(new ClassAtom(protectedArea, x1));
        setOfAtoms.add(new RoleAtom(locatedIn, x0, x1));
        Set<Variable> avarSet = Sets.newHashSet(x0, x1); // all!
        AvarCQ q0 = new AvarCQ(avarSet, setOfAtoms);

        QueryCaller.runAvarCQ(q0, Thread.currentThread().getStackTrace()[1].getMethodName(), "envo01");
    }

    public static void runENVO_nuclearPowerPlant() throws CQException, Exception {
        Signature sigma = CQContext.getINSTANCE().getOriginalSignature();

        Variable x0 = new Variable(0);
        Variable x1 = new Variable(1);

        OWLClass nuclearPowerPlant = sigma.getCls("owlENVO_00002271");
        OWLClass foodShop = sigma.getCls("owlENVO_00002222");
        OWLObjectProperty adjacentTo = sigma.getRole("owlENVO_adjacent_to");

        Set<Atom> setOfAtoms = new HashSet<>();
        setOfAtoms.add(new ClassAtom(nuclearPowerPlant, x0));
        setOfAtoms.add(new ClassAtom(foodShop, x1));
        setOfAtoms.add(new RoleAtom(adjacentTo, x0, x1));
        Set<Variable> avarSet = Sets.newHashSet(x1);
        RootedCQ q0 = new RootedCQ(avarSet, setOfAtoms);

        QueryCaller.runRootedCQ(q0, Thread.currentThread().getStackTrace()[1].getMethodName(), "envo04");
    }

    public static void runENVO_river() throws CQException, Exception {
        Signature sigma = CQContext.getINSTANCE().getOriginalSignature();

        Variable x0 = new Variable(0);
        Variable x1 = new Variable(1);

        OWLClass river = sigma.getCls("owlENVO_00000022");
        OWLClass reservoir = sigma.getCls("owlENVO_00000025");
        OWLObjectProperty adjacentTo = sigma.getRole("owlENVO_adjacent_to");
        OWLObjectProperty derivesFrom = sigma.getRole("owlENVO_derives_from");

        Set<Atom> setOfAtoms = new HashSet<>();
        setOfAtoms.add(new ClassAtom(river, x0));
        setOfAtoms.add(new ClassAtom(reservoir, x1));
        setOfAtoms.add(new RoleAtom(adjacentTo, x0, x1));
        setOfAtoms.add(new RoleAtom(derivesFrom, x0, x1));
        Set<Variable> avarSet = Sets.newHashSet(x0);
        RootedCQ q0 = new RootedCQ(avarSet, setOfAtoms);

        QueryCaller.runRootedCQ(q0, Thread.currentThread().getStackTrace()[1].getMethodName(), "envo05");
    }

    public static void runENVO_mountainRange() throws CQException, Exception {
        Signature sigma = CQContext.getINSTANCE().getOriginalSignature();

        Variable x0 = new Variable(0);
        Variable x1 = new Variable(1);
        Variable x2 = new Variable(2);
        Variable x3 = new Variable(3);
        Variable x4 = new Variable(4);

        OWLClass mountainRange = sigma.getCls("owlENVO_00000080");
        OWLClass mountain = sigma.getCls("owlENVO_00000081");
        OWLClass peninsula = sigma.getCls("owlENVO_00000305");
        OWLClass ocean = sigma.getCls("owlENVO_00000015");
        OWLClass island = sigma.getCls("owlENVO_00000098");
        OWLObjectProperty hasPart = sigma.getRole("owlENVO_has_part");
        OWLObjectProperty adjacentTo = sigma.getRole("owlENVO_adjacent_to");
        OWLObjectProperty surroundedBy = sigma.getRole("owlENVO_surrounded_by");

        Set<Atom> setOfAtoms = new HashSet<>();
        setOfAtoms.add(new ClassAtom(mountainRange, x0));
        setOfAtoms.add(new ClassAtom(mountain, x1));
        setOfAtoms.add(new ClassAtom(peninsula, x2));
        setOfAtoms.add(new ClassAtom(ocean, x3));
        setOfAtoms.add(new ClassAtom(island, x4));
        setOfAtoms.add(new RoleAtom(hasPart, x0, x1));
        setOfAtoms.add(new RoleAtom(adjacentTo, x2, x1));
        setOfAtoms.add(new RoleAtom(adjacentTo, x2, x3));
        setOfAtoms.add(new RoleAtom(surroundedBy, x4, x3));
        Set<Variable> avarSet = Sets.newHashSet(x2);
        RootedCQ q0 = new RootedCQ(avarSet, setOfAtoms);

        QueryCaller.runRootedCQ(q0, Thread.currentThread().getStackTrace()[1].getMethodName(), "envo08");
    }

    public static void runENVO_brewery() throws CQException, Exception {
        Signature sigma = CQContext.getINSTANCE().getOriginalSignature();

        Variable x0 = new Variable(0);
        Variable x1 = new Variable(1);
        Variable x2 = new Variable(2);
        Variable x3 = new Variable(3);

        OWLClass brewery = sigma.getCls("owlENVO_00003885");
        OWLClass city = sigma.getCls("owlENVO_00000856");
        OWLClass volcano = sigma.getCls("owlENVO_00000247");
        OWLClass plain = sigma.getCls("owlENVO_00000086");
        OWLObjectProperty locatedIn = sigma.getRole("owlENVO_located_in");
        OWLObjectProperty adjacentTo = sigma.getRole("owlENVO_adjacent_to");

        Set<Atom> setOfAtoms = new HashSet<>();
        setOfAtoms.add(new ClassAtom(brewery, x0));
        setOfAtoms.add(new ClassAtom(city, x1));
        setOfAtoms.add(new ClassAtom(volcano, x2));
        setOfAtoms.add(new ClassAtom(plain, x3));
        setOfAtoms.add(new RoleAtom(locatedIn, x0, x1));
        setOfAtoms.add(new RoleAtom(locatedIn, x1, x3));
        setOfAtoms.add(new RoleAtom(adjacentTo, x1, x2));
        setOfAtoms.add(new RoleAtom(locatedIn, x2, x3));
        Set<Variable> avarSet = Sets.newHashSet(x0, x1, x2, x3);
        AvarCQ q0 = new AvarCQ(avarSet, setOfAtoms);

        QueryCaller.runAvarCQ(q0, Thread.currentThread().getStackTrace()[1].getMethodName(), "envo02");
    }

    public static void runENVO_karst() throws CQException, Exception {
        Signature sigma = CQContext.getINSTANCE().getOriginalSignature();

        Variable x0 = new Variable(0);
        Variable x1 = new Variable(1);
        Variable x2 = new Variable(2);

        OWLClass polygonalKarst = sigma.getCls("owlENVO_00000335");
        OWLClass karst = sigma.getCls("owlENVO_00000175");
        OWLClass natlGeopolEntity = sigma.getCls("owlENVO_00000009");
        OWLObjectProperty locatedIn = sigma.getRole("owlENVO_located_in");

        Set<Atom> setOfAtoms = new HashSet<>();
        setOfAtoms.add(new ClassAtom(polygonalKarst, x0));
        setOfAtoms.add(new ClassAtom(karst, x1));
        setOfAtoms.add(new ClassAtom(natlGeopolEntity, x2));
        setOfAtoms.add(new RoleAtom(locatedIn, x0, x2));
        setOfAtoms.add(new RoleAtom(locatedIn, x1, x2));
        Set<Variable> avarSet = Sets.newHashSet(x0);
        RootedCQ q0 = new RootedCQ(avarSet, setOfAtoms);

        QueryCaller.runRootedCQ(q0, Thread.currentThread().getStackTrace()[1].getMethodName(), "envo10");
    }

    public static void runENVO_iceberg() throws CQException, Exception {
        Signature sigma = CQContext.getINSTANCE().getOriginalSignature();

        Variable x0 = new Variable(0);
        Variable x1 = new Variable(1);

        OWLClass iceberg = sigma.getCls("owlENVO_00000298");
        OWLClass water = sigma.getCls("owlENVO_00002006");
        OWLObjectProperty derivesFrom = sigma.getRole("owlENVO_derives_from");

        Set<Atom> setOfAtoms = new HashSet<>();
        setOfAtoms.add(new ClassAtom(iceberg, x0));
        setOfAtoms.add(new ClassAtom(water, x1));
        setOfAtoms.add(new RoleAtom(derivesFrom, x0, x1));
        setOfAtoms.add(new RoleAtom(derivesFrom, x1, x0));
        Set<Variable> avarSet = Sets.newHashSet(x0);
        RootedCQ q0 = new RootedCQ(avarSet, setOfAtoms);

        QueryCaller.runRootedCQ(q0, Thread.currentThread().getStackTrace()[1].getMethodName(), "envo06");
    }

    public static void runENVO_volcanicCrater() throws CQException, Exception {
        Signature sigma = CQContext.getINSTANCE().getOriginalSignature();

        Variable x0 = new Variable(0);
        Variable x1 = new Variable(1);
        Variable x2 = new Variable(2);

        OWLClass volcano = sigma.getCls("owlENVO_00000247");
        OWLClass volcanicCrater = sigma.getCls("owlENVO_00000246");
        OWLClass islandArc = sigma.getCls("owlENVO_00000353");
        OWLObjectProperty locatedIn = sigma.getRole("owlENVO_located_in");
        OWLObjectProperty hasPart = sigma.getRole("owlENVO_has_part");

        Set<Atom> setOfAtoms = new HashSet<>();
        setOfAtoms.add(new ClassAtom(volcano, x0));
        setOfAtoms.add(new ClassAtom(islandArc, x1));
        setOfAtoms.add(new ClassAtom(volcanicCrater, x2));
        setOfAtoms.add(new RoleAtom(locatedIn, x0, x1));
        setOfAtoms.add(new RoleAtom(hasPart, x0, x2));
        Set<Variable> avarSet = Sets.newHashSet(x0);
        RootedCQ q0 = new RootedCQ(avarSet, setOfAtoms);

        QueryCaller.runRootedCQ(q0, Thread.currentThread().getStackTrace()[1].getMethodName(), "envo09");
    }

    public static void runENVO_glacier() throws CQException, Exception {
        Signature sigma = CQContext.getINSTANCE().getOriginalSignature();

        Variable x0 = new Variable(0);
        Variable x1 = new Variable(1);
        Variable x2 = new Variable(2);
        Variable x3 = new Variable(3);

        OWLClass mountain = sigma.getCls("owlENVO_00000081");
        OWLClass mountainRange = sigma.getCls("owlENVO_00000080");
        OWLClass glacier = sigma.getCls("owlENVO_00000133");
        OWLClass volcano = sigma.getCls("owlENVO_00000247");
        OWLObjectProperty locatedIn = sigma.getRole("owlENVO_located_in");
        OWLObjectProperty hasPart = sigma.getRole("owlENVO_has_part");

        Set<Atom> setOfAtoms = new HashSet<>();
        setOfAtoms.add(new ClassAtom(mountain, x0));
        setOfAtoms.add(new ClassAtom(mountainRange, x1));
        setOfAtoms.add(new ClassAtom(glacier, x2));
        setOfAtoms.add(new ClassAtom(volcano, x3));
        setOfAtoms.add(new RoleAtom(hasPart, x0, x2));
        setOfAtoms.add(new RoleAtom(locatedIn, x0, x1));
        setOfAtoms.add(new RoleAtom(hasPart, x1, x3));
        Set<Variable> avarSet = Sets.newHashSet(x0, x1);
        TqCQ q0 = new TqCQ(avarSet, setOfAtoms);

        QueryCaller.runTqCQ(q0, Thread.currentThread().getStackTrace()[1].getMethodName(), "envo03");
    }

    public static void runSO_biologicalRegion() throws CQException, Exception {
        Signature sigma = CQContext.getINSTANCE().getOriginalSignature();

        Variable x0 = new Variable(0);
        Variable x1 = new Variable(1);
        Variable x2 = new Variable(2);

        OWLClass biologicalRegion = sigma.getCls("owlSO_0001411");
        OWLClass cds = sigma.getCls("owlSO_0000316");
        OWLClass polypeptide = sigma.getCls("owlSO_0000104");
        OWLObjectProperty partOf = sigma.getRole("owlSEQUENCE_part_of");
        OWLObjectProperty transcribedTo = sigma.getRole("owlSEQUENCE_transcribed_to");

        Set<Atom> setOfAtoms = new HashSet<>();
        setOfAtoms.add(new ClassAtom(biologicalRegion, x0));
        setOfAtoms.add(new ClassAtom(cds, x1));
        setOfAtoms.add(new ClassAtom(polypeptide, x2));
        setOfAtoms.add(new RoleAtom(partOf, x0, x2));
        setOfAtoms.add(new RoleAtom(transcribedTo, x1, x2));
        Set<Variable> avarSet = Sets.newHashSet(x1);
        RootedCQ q0 = new RootedCQ(avarSet, setOfAtoms);

        QueryCaller.runRootedCQ(q0, Thread.currentThread().getStackTrace()[1].getMethodName(), "so09");
    }

    public static void runSO_alanine() throws CQException, Exception {
        Signature sigma = CQContext.getINSTANCE().getOriginalSignature();

        Variable x0 = new Variable(0);
        Variable x1 = new Variable(1);
        Variable x2 = new Variable(2);

        OWLClass biologicalRegion = sigma.getCls("owlSO_0001411");
        OWLClass alanine = sigma.getCls("owlSO_0001435");
        OWLClass polypeptide = sigma.getCls("owlSO_0000104");
        OWLObjectProperty partOf = sigma.getRole("owlSEQUENCE_part_of");

        Set<Atom> setOfAtoms = new HashSet<>();
        setOfAtoms.add(new ClassAtom(biologicalRegion, x0));
        setOfAtoms.add(new ClassAtom(alanine, x1));
        setOfAtoms.add(new ClassAtom(polypeptide, x2));
        setOfAtoms.add(new RoleAtom(partOf, x0, x2));
        setOfAtoms.add(new RoleAtom(partOf, x1, x2));
        Set<Variable> avarSet = Sets.newHashSet(x1);
        RootedCQ q0 = new RootedCQ(avarSet, setOfAtoms);

        QueryCaller.runRootedCQ(q0, Thread.currentThread().getStackTrace()[1].getMethodName(), "so10");
    }

    public static void runSO_insertionBreakpoint() throws CQException, Exception {
        Signature sigma = CQContext.getINSTANCE().getOriginalSignature();

        Variable x0 = new Variable(0);
        Variable x1 = new Variable(1);
        Variable x2 = new Variable(2);

        OWLClass insertionBreakpoint = sigma.getCls("owlSO_0001414");
        OWLClass spliceJunction = sigma.getCls("owlSO_0001421");
        OWLClass genome = sigma.getCls("owlSO_0001026");
        OWLObjectProperty partOf = sigma.getRole("owlSEQUENCE_part_of");

        Set<Atom> setOfAtoms = new HashSet<>();
        setOfAtoms.add(new ClassAtom(insertionBreakpoint, x0));
        setOfAtoms.add(new ClassAtom(spliceJunction, x1));
        setOfAtoms.add(new ClassAtom(genome, x2));
        setOfAtoms.add(new RoleAtom(partOf, x0, x2));
        setOfAtoms.add(new RoleAtom(partOf, x1, x2));
        Set<Variable> avarSet = Sets.newHashSet(x0);
        RootedCQ q0 = new RootedCQ(avarSet, setOfAtoms);

        QueryCaller.runRootedCQ(q0, Thread.currentThread().getStackTrace()[1].getMethodName(), "so07");
    }

    public static void runSO_mobileElementInsertion() throws CQException, Exception {
        Signature sigma = CQContext.getINSTANCE().getOriginalSignature();

        Variable x0 = new Variable(0);
        Variable x1 = new Variable(1);

        OWLClass mobileElementInsertion = sigma.getCls("owlSO_0001837");
        OWLClass cointegratedPlasma = sigma.getCls("owlSO_0001045");
        OWLObjectProperty contains = sigma.getRole("owlSEQUENCE_contains");

        Set<Atom> setOfAtoms = new HashSet<>();
        setOfAtoms.add(new ClassAtom(mobileElementInsertion, x0));
        setOfAtoms.add(new ClassAtom(cointegratedPlasma, x1));
        setOfAtoms.add(new RoleAtom(contains, x0, x1));
        Set<Variable> avarSet = Sets.newHashSet(x1);
        RootedCQ q0 = new RootedCQ(avarSet, setOfAtoms);

        QueryCaller.runRootedCQ(q0, Thread.currentThread().getStackTrace()[1].getMethodName(), "so04");
    }

    public static void runSO_spliceJunction() throws CQException, Exception {
        Signature sigma = CQContext.getINSTANCE().getOriginalSignature();

        Variable x0 = new Variable(0);
        Variable x1 = new Variable(1);

        OWLClass spliceJunction = sigma.getCls("owlSO_0001421");
        OWLClass chloroplastChromosome = sigma.getCls("owlSO_0000820");
        OWLObjectProperty adjacentTo = sigma.getRole("owlSEQUENCE_adjacent_to");
        OWLObjectProperty partOf = sigma.getRole("owlSEQUENCE_part_of");

        Set<Atom> setOfAtoms = new HashSet<>();
        setOfAtoms.add(new ClassAtom(spliceJunction, x0));
        setOfAtoms.add(new ClassAtom(chloroplastChromosome, x1));
        setOfAtoms.add(new RoleAtom(adjacentTo, x0, x0));
        setOfAtoms.add(new RoleAtom(partOf, x0, x1));
        Set<Variable> avarSet = Sets.newHashSet(x1);
        RootedCQ q0 = new RootedCQ(avarSet, setOfAtoms);

        QueryCaller.runRootedCQ(q0, Thread.currentThread().getStackTrace()[1].getMethodName(), "so05");
    }

    public static void runSO_epitope() throws CQException, Exception {
        Signature sigma = CQContext.getINSTANCE().getOriginalSignature();

        Variable x0 = new Variable(0);
        Variable x1 = new Variable(1);
        Variable x2 = new Variable(2);
        Variable x3 = new Variable(3);
        Variable x4 = new Variable(4);

        OWLClass polypepLigandContact = sigma.getCls("owlSO_0001105");
        OWLClass polypepMetalContact = sigma.getCls("owlSO_0001092");
        OWLClass epitope = sigma.getCls("owlSO_0001018");
        OWLClass polypeptide = sigma.getCls("owlSO_0000104");
        OWLClass crypticGene = sigma.getCls("owlSO_0001431");
        OWLObjectProperty hasOrigin = sigma.getRole("owlSEQUENCE_has_origin");
        OWLObjectProperty partOf = sigma.getRole("owlSEQUENCE_part_of");

        Set<Atom> setOfAtoms = new HashSet<>();
        setOfAtoms.add(new ClassAtom(polypepLigandContact, x0));
        setOfAtoms.add(new ClassAtom(polypepMetalContact, x1));
        setOfAtoms.add(new ClassAtom(epitope, x2));
        setOfAtoms.add(new ClassAtom(polypeptide, x3));
        setOfAtoms.add(new ClassAtom(crypticGene, x4));
        setOfAtoms.add(new RoleAtom(partOf, x0, x3));
        setOfAtoms.add(new RoleAtom(partOf, x1, x3));
        setOfAtoms.add(new RoleAtom(hasOrigin, x1, x4));
        setOfAtoms.add(new RoleAtom(hasOrigin, x2, x4));
        Set<Variable> avarSet = Sets.newHashSet(x1);
        RootedCQ q0 = new RootedCQ(avarSet, setOfAtoms);

        QueryCaller.runRootedCQ(q0, Thread.currentThread().getStackTrace()[1].getMethodName(), "so08");
    }

    public static void runSO_proviralGene() throws CQException, Exception {
        Signature sigma = CQContext.getINSTANCE().getOriginalSignature();

        Variable x0 = new Variable(0);
        Variable x1 = new Variable(1);
        Variable x2 = new Variable(2);
        Variable x3 = new Variable(3);

        OWLClass proviralGene = sigma.getCls("owlSO_0000099");
        OWLClass geneArray = sigma.getCls("owlSO_0005851");
        OWLClass cpgIsland = sigma.getCls("owlSO_0000307");
        OWLClass retrogene = sigma.getCls("owlSO_0001219");
        OWLObjectProperty memberOf = sigma.getRole("owlSEQUENCE_member_of");
        OWLObjectProperty hasPart = sigma.getRole("owlSEQUENCE_has_part");
        OWLObjectProperty contains = sigma.getRole("owlSEQUENCE_contains");

        Set<Atom> setOfAtoms = new HashSet<>();
        setOfAtoms.add(new ClassAtom(proviralGene, x0));
        setOfAtoms.add(new ClassAtom(geneArray, x1));
        setOfAtoms.add(new ClassAtom(cpgIsland, x2));
        setOfAtoms.add(new ClassAtom(retrogene, x3));
        setOfAtoms.add(new RoleAtom(hasPart, x0, x2));
        setOfAtoms.add(new RoleAtom(memberOf, x0, x1));
        setOfAtoms.add(new RoleAtom(contains, x1, x3));
        Set<Variable> avarSet = Sets.newHashSet(x0, x1);
        TqCQ q0 = new TqCQ(avarSet, setOfAtoms);

        QueryCaller.runTqCQ(q0, Thread.currentThread().getStackTrace()[1].getMethodName(), "so03");
    }

    public static void runSO_pipBox() throws CQException, Exception {
        Signature sigma = CQContext.getINSTANCE().getOriginalSignature();

        Variable x0 = new Variable(0);
        Variable x1 = new Variable(1);
        Variable x2 = new Variable(2);
        Variable x3 = new Variable(3);

        OWLClass mtGene = sigma.getCls("owlSO_0000088");
        OWLClass geneSubarray = sigma.getCls("owlSO_0005852");
        OWLClass cyanelleGene = sigma.getCls("owlSO_0000094");
        OWLClass pipBox = sigma.getCls("owlSO_0001810");
        OWLObjectProperty contains = sigma.getRole("owlSEQUENCE_contains");
        OWLObjectProperty transcribedTo = sigma.getRole("owlSEQUENCE_transcribed_to");
        OWLObjectProperty adjacentTo = sigma.getRole("owlSEQUENCE_adjacent_to");

        Set<Atom> setOfAtoms = new HashSet<>();
        setOfAtoms.add(new ClassAtom(mtGene, x0));
        setOfAtoms.add(new ClassAtom(geneSubarray, x1));
        setOfAtoms.add(new ClassAtom(cyanelleGene, x2));
        setOfAtoms.add(new ClassAtom(pipBox, x3));
        setOfAtoms.add(new RoleAtom(adjacentTo, x0, x1));
        setOfAtoms.add(new RoleAtom(transcribedTo, x1, x3));
        setOfAtoms.add(new RoleAtom(contains, x1, x2));
        setOfAtoms.add(new RoleAtom(transcribedTo, x2, x3));
        Set<Variable> avarSet = Sets.newHashSet(x0, x1, x2, x3);
        AvarCQ q0 = new AvarCQ(avarSet, setOfAtoms);

        QueryCaller.runAvarCQ(q0, Thread.currentThread().getStackTrace()[1].getMethodName(), "so02");
    }

    public static void runSO_chromosomeArm() throws CQException, Exception {
        Signature sigma = CQContext.getINSTANCE().getOriginalSignature();

        Variable x0 = new Variable(0);
        Variable x1 = new Variable(1);

        OWLClass chromosomeArm = sigma.getCls("owlSO_0000105");
        OWLClass leucoplastChromosome = sigma.getCls("owlSO_0000823");
        OWLObjectProperty partOf = sigma.getRole("owlSEQUENCE_part_of");

        Set<Atom> setOfAtoms = new HashSet<>();
        setOfAtoms.add(new ClassAtom(chromosomeArm, x0));
        setOfAtoms.add(new ClassAtom(leucoplastChromosome, x1));
        setOfAtoms.add(new RoleAtom(partOf, x0, x1));
        Set<Variable> avarSet = Sets.newHashSet(x0, x1);
        AvarCQ q0 = new AvarCQ(avarSet, setOfAtoms);

        QueryCaller.runAvarCQ(q0, Thread.currentThread().getStackTrace()[1].getMethodName(), "so01");
    }

    public static void runSO_engineeredFusionGene() throws CQException, Exception {
        Signature sigma = CQContext.getINSTANCE().getOriginalSignature();

        Variable x0 = new Variable(0);
        Variable x1 = new Variable(1);

        OWLClass engineeredFusionGene = sigma.getCls("owlSO_0000288");
        OWLClass geneSilencedByDNAmethylation = sigma.getCls("owlSO_0000129");
        OWLObjectProperty variantOf = sigma.getRole("owlSEQUENCE_variant_of");

        Set<Atom> setOfAtoms = new HashSet<>();
        setOfAtoms.add(new ClassAtom(engineeredFusionGene, x0));
        setOfAtoms.add(new ClassAtom(geneSilencedByDNAmethylation, x1));
        setOfAtoms.add(new RoleAtom(variantOf, x0, x1));
        setOfAtoms.add(new RoleAtom(variantOf, x1, x0));
        Set<Variable> avarSet = Sets.newHashSet(x0);
        RootedCQ q0 = new RootedCQ(avarSet, setOfAtoms);

        QueryCaller.runRootedCQ(q0, Thread.currentThread().getStackTrace()[1].getMethodName(), "so06");
    }

    public static void runNotGalen_endocarditis() throws CQException, Exception {
        Signature sigma = CQContext.getINSTANCE().getOriginalSignature();

        Variable x0 = new Variable(0);
        Variable x1 = new Variable(1);

        OWLClass patient = sigma.getCls("#Patient");
        OWLClass endocarditis = sigma.getCls("#Endocarditis");
        OWLObjectProperty shows = sigma.getRole("shows");

        Set<Atom> setOfAtoms = new HashSet<>();
        setOfAtoms.add(new ClassAtom(patient, x0));
        setOfAtoms.add(new ClassAtom(endocarditis, x1));
        setOfAtoms.add(new RoleAtom(shows, x0, x1));
        Set<Variable> avarSet = Sets.newHashSet(x0, x1);
        AvarCQ q0 = new AvarCQ(avarSet, setOfAtoms);

        QueryCaller.runAvarCQ(q0, Thread.currentThread().getStackTrace()[1].getMethodName(), "ng01");
    }

    public static void runNotGalen_tissue() throws CQException, Exception {
        Signature sigma = CQContext.getINSTANCE().getOriginalSignature();

        Variable x0 = new Variable(0);
        Variable x1 = new Variable(1);
        Variable x2 = new Variable(2);
        Variable x3 = new Variable(3);

        OWLClass doctor = sigma.getCls("#Doctor");
        OWLClass surgery = sigma.getCls("#Surgery");
        OWLClass tissue = sigma.getCls("#Tissue");
        OWLClass internalOrgan = sigma.getCls("#InternalOrgan");
        OWLObjectProperty hasPersonPerforming = sigma.getRole("hasPersonPerforming");
        OWLObjectProperty actsOn = sigma.getRole("actsOn");
        OWLObjectProperty hasAlphaConnection = sigma.getRole("hasAlphaConnection");

        Set<Atom> setOfAtoms = new HashSet<>();
        setOfAtoms.add(new ClassAtom(doctor, x0));
        setOfAtoms.add(new ClassAtom(surgery, x1));
        setOfAtoms.add(new ClassAtom(tissue, x2));
        setOfAtoms.add(new ClassAtom(internalOrgan, x3));
        setOfAtoms.add(new RoleAtom(hasPersonPerforming, x1, x0));
        setOfAtoms.add(new RoleAtom(actsOn, x1, x2));
        setOfAtoms.add(new RoleAtom(actsOn, x1, x3));
        setOfAtoms.add(new RoleAtom(hasAlphaConnection, x2, x3));
        Set<Variable> avarSet = Sets.newHashSet(x0, x1, x2, x3);
        AvarCQ q0 = new AvarCQ(avarSet, setOfAtoms);

        QueryCaller.runAvarCQ(q0, Thread.currentThread().getStackTrace()[1].getMethodName(), "ng02");
    }

    public static void runNotGalen_giardia() throws CQException, Exception {
        Signature sigma = CQContext.getINSTANCE().getOriginalSignature();

        Variable x0 = new Variable(0);
        Variable x1 = new Variable(1);

        OWLClass aminoglycoside = sigma.getCls("#Aminoglycoside");
        OWLClass giardia = sigma.getCls("#Giardia");
        OWLObjectProperty actsOn = sigma.getRole("actsOn");

        Set<Atom> setOfAtoms = new HashSet<>();
        setOfAtoms.add(new ClassAtom(aminoglycoside, x0));
        setOfAtoms.add(new ClassAtom(giardia, x1));
        setOfAtoms.add(new RoleAtom(actsOn, x0, x1));
        Set<Variable> avarSet = Sets.newHashSet(x1);
        RootedCQ q0 = new RootedCQ(avarSet, setOfAtoms);

        QueryCaller.runRootedCQ(q0, Thread.currentThread().getStackTrace()[1].getMethodName(), "ng04");
    }

    public static void runNotGalen_tetracycline() throws CQException, Exception {
        Signature sigma = CQContext.getINSTANCE().getOriginalSignature();

        Variable x0 = new Variable(0);
        Variable x1 = new Variable(1);
        Variable x2 = new Variable(2);

        OWLClass protein = sigma.getCls("#Protein");
        OWLClass internalOrgan = sigma.getCls("#InternalOrgan");
        OWLClass Tetracycline = sigma.getCls("#Tetracycline");
        OWLObjectProperty isActedOnSpecificallyBy = sigma.getRole("isActedOnSpecificallyBy");
        OWLObjectProperty contains = sigma.getRole("contains");

        Set<Atom> setOfAtoms = new HashSet<>();
        setOfAtoms.add(new ClassAtom(protein, x0));
        setOfAtoms.add(new ClassAtom(internalOrgan, x1));
        setOfAtoms.add(new ClassAtom(Tetracycline, x2));
        setOfAtoms.add(new RoleAtom(contains, x0, x2));
        setOfAtoms.add(new RoleAtom(isActedOnSpecificallyBy, x1, x2));
        Set<Variable> avarSet = Sets.newHashSet(x0);
        RootedCQ q0 = new RootedCQ(avarSet, setOfAtoms);

        QueryCaller.runRootedCQ(q0, Thread.currentThread().getStackTrace()[1].getMethodName(), "ng07");
    }

    public static void runNotGalen_voluntaryMovement() throws CQException, Exception {

        Signature sigma = CQContext.getINSTANCE().getOriginalSignature();

        Variable x0 = new Variable(0);
        Variable x1 = new Variable(1);

        OWLClass pathologyConstruct = sigma.getCls("#PathologyConstruct");
        OWLClass voluntaryMovement = sigma.getCls("#VoluntaryMovement");
        OWLObjectProperty isCausalAgentOf = sigma.getRole("isCausalAgentOf");
        OWLObjectProperty occursDuring = sigma.getRole("occursDuring");

        Set<Atom> setOfAtoms = new HashSet<>();
        setOfAtoms.add(new ClassAtom(pathologyConstruct, x0));
        setOfAtoms.add(new ClassAtom(voluntaryMovement, x1));
        setOfAtoms.add(new RoleAtom(isCausalAgentOf, x0, x0));
        setOfAtoms.add(new RoleAtom(occursDuring, x0, x1));
        Set<Variable> avarSet = Sets.newHashSet(x1);
        RootedCQ q0 = new RootedCQ(avarSet, setOfAtoms);

        QueryCaller.runRootedCQ(q0, Thread.currentThread().getStackTrace()[1].getMethodName(), "ng06");
    }

    public static void runNotGalen_duct() throws CQException, Exception {

        Signature sigma = CQContext.getINSTANCE().getOriginalSignature();

        Variable x0 = new Variable(0);
        Variable x1 = new Variable(1);

        OWLClass Duct = sigma.getCls("#Duct");
        OWLClass InternalOrgan = sigma.getCls("#InternalOrgan");
        OWLObjectProperty isBlindPouchDivisionOf = sigma.getRole("isBlindPouchDivisionOf");
        OWLObjectProperty carriesTo = sigma.getRole("carriesTo");

        Set<Atom> setOfAtoms = new HashSet<>();
        setOfAtoms.add(new ClassAtom(Duct, x0));
        setOfAtoms.add(new ClassAtom(InternalOrgan, x1));
        setOfAtoms.add(new RoleAtom(isBlindPouchDivisionOf, x0, x1));
        setOfAtoms.add(new RoleAtom(carriesTo, x0, x1));
        Set<Variable> avarSet = Sets.newHashSet(x0);
        RootedCQ q0 = new RootedCQ(avarSet, setOfAtoms);

        QueryCaller.runRootedCQ(q0, Thread.currentThread().getStackTrace()[1].getMethodName(), "ng05");
    }

    public static void runNotGalen_teichoicAcid() throws CQException, Exception {
        Signature sigma = CQContext.getINSTANCE().getOriginalSignature();

        Variable x0 = new Variable(0);
        Variable x1 = new Variable(1);
        Variable x2 = new Variable(2);
        Variable x3 = new Variable(3);
        Variable x4 = new Variable(4);

        OWLClass sulphonamide = sigma.getCls("#Sulphonamide");
        OWLClass enzyme = sigma.getCls("#NAMEDEnzyme");
        OWLClass teichoicAcid = sigma.getCls("#TeichoicAcid");
        OWLClass tumorMarkerRole = sigma.getCls("#TumorMarkerRole");
        OWLClass liver = sigma.getCls("#Liver");
        OWLObjectProperty serves = sigma.getRole("serves");
        OWLObjectProperty actsOn = sigma.getRole("actsOn");

        Set<Atom> setOfAtoms = new HashSet<>();
        setOfAtoms.add(new ClassAtom(sulphonamide, x0));
        setOfAtoms.add(new ClassAtom(enzyme, x1));
        setOfAtoms.add(new ClassAtom(teichoicAcid, x2));
        setOfAtoms.add(new ClassAtom(tumorMarkerRole, x3));
        setOfAtoms.add(new ClassAtom(liver, x4));
        setOfAtoms.add(new RoleAtom(serves, x0, x3));
        setOfAtoms.add(new RoleAtom(serves, x1, x3));
        setOfAtoms.add(new RoleAtom(actsOn, x1, x4));
        setOfAtoms.add(new RoleAtom(actsOn, x2, x4));
        Set<Variable> avarSet = Sets.newHashSet(x1);
        RootedCQ q0 = new RootedCQ(avarSet, setOfAtoms);

        QueryCaller.runRootedCQ(q0, Thread.currentThread().getStackTrace()[1].getMethodName(), "ng08");
    }

    public static void runNotGalen_alcohol() throws CQException, Exception {
        Signature sigma = CQContext.getINSTANCE().getOriginalSignature();

        Variable x0 = new Variable(0);
        Variable x1 = new Variable(1);
        Variable x2 = new Variable(2);
        Variable x3 = new Variable(3);

        OWLClass triglyceride = sigma.getCls("#Triglyceride");
        OWLClass alcohol = sigma.getCls("#Alcohol");
        OWLClass investigationAct = sigma.getCls("#InvestigationAct");
        OWLClass careAct = sigma.getCls("#CareAct");
        OWLObjectProperty serves = sigma.getRole("serves");
        OWLObjectProperty hasDissolvedWithin = sigma.getRole("hasDissolvedWithin");

        Set<Atom> setOfAtoms = new HashSet<>();
        setOfAtoms.add(new ClassAtom(triglyceride, x0));
        setOfAtoms.add(new ClassAtom(alcohol, x1));
        setOfAtoms.add(new ClassAtom(investigationAct, x2));
        setOfAtoms.add(new ClassAtom(careAct, x3));
        setOfAtoms.add(new RoleAtom(hasDissolvedWithin, x0, x1));
        setOfAtoms.add(new RoleAtom(serves, x0, x2));
        setOfAtoms.add(new RoleAtom(serves, x1, x3));
        Set<Variable> avarSet = Sets.newHashSet(x0, x1);
        TqCQ q0 = new TqCQ(avarSet, setOfAtoms);

        QueryCaller.runTqCQ(q0, Thread.currentThread().getStackTrace()[1].getMethodName(), "ng03");
    }

    public static void runNotGalen_brain() throws CQException, Exception {
        Signature sigma = CQContext.getINSTANCE().getOriginalSignature();

        Variable x0 = new Variable(0);
        Variable x1 = new Variable(1);
        Variable x2 = new Variable(2);

        OWLClass bodyStructure = sigma.getCls("#BodyStructure");
        OWLClass intrinsicallyNormalBodyStructure = sigma.getCls("#IntrinsicallyNormalBodyStructure");
        OWLClass brain = sigma.getCls("#Brain");
        OWLObjectProperty isBetaConnectionOf = sigma.getRole("isBetaConnectionOf");

        Set<Atom> setOfAtoms = new HashSet<>();
        setOfAtoms.add(new ClassAtom(bodyStructure, x0));
        setOfAtoms.add(new ClassAtom(intrinsicallyNormalBodyStructure, x1));
        setOfAtoms.add(new ClassAtom(brain, x2));
        setOfAtoms.add(new RoleAtom(isBetaConnectionOf, x0, x2));
        setOfAtoms.add(new RoleAtom(isBetaConnectionOf, x1, x2));
        Set<Variable> avarSet = Sets.newHashSet(x0);
        RootedCQ q0 = new RootedCQ(avarSet, setOfAtoms);

        QueryCaller.runRootedCQ(q0, Thread.currentThread().getStackTrace()[1].getMethodName(), "ng10");
    }

    public static void runNotGalen_bordatella() throws CQException, Exception {
        Signature sigma = CQContext.getINSTANCE().getOriginalSignature();

        Variable x0 = new Variable(0);
        Variable x1 = new Variable(1);
        Variable x2 = new Variable(2);

        OWLClass diseaseState = sigma.getCls("#DiseaseState");
        OWLClass bordatella = sigma.getCls("#Bordatella");
        OWLClass nonNormalCondition = sigma.getCls("#NonNormalCondition");
        OWLObjectProperty hasCausalAgent = sigma.getRole("hasCausalAgent");
        OWLObjectProperty isCausallyLinkedTo = sigma.getRole("IsCausallyLinkedTo");

        Set<Atom> setOfAtoms = new HashSet<>();
        setOfAtoms.add(new ClassAtom(diseaseState, x0));
        setOfAtoms.add(new ClassAtom(bordatella, x1));
        setOfAtoms.add(new ClassAtom(nonNormalCondition, x2));
        setOfAtoms.add(new RoleAtom(hasCausalAgent, x0, x1));
        setOfAtoms.add(new RoleAtom(isCausallyLinkedTo, x0, x2));
        Set<Variable> avarSet = Sets.newHashSet(x0);
        RootedCQ q0 = new RootedCQ(avarSet, setOfAtoms);

        QueryCaller.runRootedCQ(q0, Thread.currentThread().getStackTrace()[1].getMethodName(), "ng09");
    }

    public static void runFBbi_negativeStaining() throws CQException, Exception {
        Signature sigma = CQContext.getINSTANCE().getOriginalSignature();

        Variable x0 = new Variable(0);
        Variable x1 = new Variable(1);

        OWLClass microscopy = sigma.getCls("#FBbi_00000241");
        OWLClass negativeStaining = sigma.getCls("#FBbi_00000399");
        OWLObjectProperty makesUseOf = sigma.getRole("#FBbi_00000346");

        Set<Atom> setOfAtoms = new HashSet<>();
        setOfAtoms.add(new ClassAtom(microscopy, x0));
        setOfAtoms.add(new ClassAtom(negativeStaining, x1));
        setOfAtoms.add(new RoleAtom(makesUseOf, x0, x1));
        Set<Variable> avarSet = Sets.newHashSet(x0, x1);
        AvarCQ q0 = new AvarCQ(avarSet, setOfAtoms);

        QueryCaller.runAvarCQ(q0, Thread.currentThread().getStackTrace()[1].getMethodName(), "fbbi01");
    }

    public static void runFBbi_avarOptiTest() throws CQException, Exception {
        Signature sigma = CQContext.getINSTANCE().getOriginalSignature();

        Variable x0 = new Variable(0);
        Variable x1 = new Variable(1);

        OWLClass microscopy = sigma.getCls("#FBbi_00000241");
        OWLClass negativeStaining = sigma.getCls("#FBbi_00000399");
        OWLObjectProperty makesUseOf = sigma.getRole("#FBbi_00000346");

        Set<Atom> setOfAtoms = new HashSet<>();
        setOfAtoms.add(new ClassAtom(microscopy, x0));
        setOfAtoms.add(new ClassAtom(negativeStaining, x0));
        setOfAtoms.add(new ClassAtom(negativeStaining, x1));
        setOfAtoms.add(new RoleAtom(makesUseOf, x0, x1));
        Set<Variable> avarSet = Sets.newHashSet(x0, x1);
        AvarCQ q0 = new AvarCQ(avarSet, setOfAtoms);

        QueryCaller.runAvarCQ(q0, Thread.currentThread().getStackTrace()[1].getMethodName(), "fbbi01");
    }

    public static void runFBbi_eyfp() throws CQException, Exception {
        Signature sigma = CQContext.getINSTANCE().getOriginalSignature();

        Variable x0 = new Variable(0);
        Variable x1 = new Variable(1);
        Variable x2 = new Variable(2);
        Variable x3 = new Variable(3);

        OWLClass diagram = sigma.getCls("#FBbi_00000033");
        OWLClass microscopy = sigma.getCls("#FBbi_00000241");
        OWLClass geneticallyEncodedTag = sigma.getCls("#FBbi_00000079");
        OWLClass eyfp = sigma.getCls("#FBbi_00000084");
        OWLObjectProperty makesUseOf = sigma.getRole("#FBbi_00000346");

        Set<Atom> setOfAtoms = new HashSet<>();
        setOfAtoms.add(new ClassAtom(diagram, x0));
        setOfAtoms.add(new ClassAtom(microscopy, x1));
        setOfAtoms.add(new ClassAtom(geneticallyEncodedTag, x2));
        setOfAtoms.add(new ClassAtom(eyfp, x3));
        setOfAtoms.add(new RoleAtom(makesUseOf, x0, x1));
        setOfAtoms.add(new RoleAtom(makesUseOf, x1, x2));
        setOfAtoms.add(new RoleAtom(makesUseOf, x1, x3));
        setOfAtoms.add(new RoleAtom(makesUseOf, x2, x3));
        Set<Variable> avarSet = Sets.newHashSet(x0, x1, x2, x3);
        AvarCQ q0 = new AvarCQ(avarSet, setOfAtoms);

        QueryCaller.runAvarCQ(q0, Thread.currentThread().getStackTrace()[1].getMethodName(), "fbbi02");
    }

    public static void runFBbi_luciferYellow() throws CQException, Exception {
        Signature sigma = CQContext.getINSTANCE().getOriginalSignature();

        Variable x0 = new Variable(0);
        Variable x1 = new Variable(1);
        Variable x2 = new Variable(2);
        Variable x3 = new Variable(3);

        OWLClass timelapseMicroscopy = sigma.getCls("#FBbi_00000249");
        OWLClass flim = sigma.getCls("#FBbi_00000368");
        OWLClass livingTissue = sigma.getCls("#FBbi_00000025");
        OWLClass luciferYellow = sigma.getCls("#FBbi_00000057");
        OWLObjectProperty makesUseOf = sigma.getRole("#FBbi_00000346");

        Set<Atom> setOfAtoms = new HashSet<>();
        setOfAtoms.add(new ClassAtom(timelapseMicroscopy, x0));
        setOfAtoms.add(new ClassAtom(flim, x1));
        setOfAtoms.add(new ClassAtom(livingTissue, x2));
        setOfAtoms.add(new ClassAtom(luciferYellow, x3));
        setOfAtoms.add(new RoleAtom(makesUseOf, x0, x1));
        setOfAtoms.add(new RoleAtom(makesUseOf, x0, x2));
        setOfAtoms.add(new RoleAtom(makesUseOf, x1, x3));
        Set<Variable> avarSet = Sets.newHashSet(x0, x1);
        TqCQ q0 = new TqCQ(avarSet, setOfAtoms);

        QueryCaller.runTqCQ(q0, Thread.currentThread().getStackTrace()[1].getMethodName(), "fbbi03");
    }

    public static void runFBbi_cryostatSectionedTissue() throws CQException, Exception {
        Signature sigma = CQContext.getINSTANCE().getOriginalSignature();

        Variable x0 = new Variable(0);
        Variable x1 = new Variable(1);

        OWLClass scanningProbeMicroscopy = sigma.getCls("#FBbi_00000344");
        OWLClass cryostatSectionedTissue = sigma.getCls("#FBbi_00000027");
        OWLObjectProperty makesUseOf = sigma.getRole("#FBbi_00000346");

        Set<Atom> setOfAtoms = new HashSet<>();
        setOfAtoms.add(new ClassAtom(scanningProbeMicroscopy, x0));
        setOfAtoms.add(new ClassAtom(cryostatSectionedTissue, x1));
        setOfAtoms.add(new RoleAtom(makesUseOf, x0, x1));
        Set<Variable> avarSet = Sets.newHashSet(x1);
        RootedCQ q0 = new RootedCQ(avarSet, setOfAtoms);

        QueryCaller.runRootedCQ(q0, Thread.currentThread().getStackTrace()[1].getMethodName(), "fbbi04");
    }

    public static void runFBbi_areaDetector() throws CQException, Exception {
        Signature sigma = CQContext.getINSTANCE().getOriginalSignature();

        Variable x0 = new Variable(0);
        Variable x1 = new Variable(1);

        OWLClass areaDetector = sigma.getCls("#FBbi_00000395");
        OWLClass avalanchePhotodiode = sigma.getCls("#FBbi_00000297");
        OWLObjectProperty makesUseOf = sigma.getRole("#FBbi_00000346");

        Set<Atom> setOfAtoms = new HashSet<>();
        setOfAtoms.add(new ClassAtom(areaDetector, x0));
        setOfAtoms.add(new ClassAtom(avalanchePhotodiode, x1));
        setOfAtoms.add(new RoleAtom(makesUseOf, x0, x1));
        setOfAtoms.add(new RoleAtom(makesUseOf, x1, x0));
        Set<Variable> avarSet = Sets.newHashSet(x0);
        RootedCQ q0 = new RootedCQ(avarSet, setOfAtoms);

        QueryCaller.runRootedCQ(q0, Thread.currentThread().getStackTrace()[1].getMethodName(), "fbbi05");
    }

    public static void runFBbi_detectionOfPhotons() throws CQException, Exception {
        Signature sigma = CQContext.getINSTANCE().getOriginalSignature();

        Variable x0 = new Variable(0);
        Variable x1 = new Variable(1);

        OWLClass detectionOfPhotons = sigma.getCls("#FBbi_00000374");
        OWLClass polarizationSensitiveDetection = sigma.getCls("#FBbi_00000352");
        OWLObjectProperty makesUseOf = sigma.getRole("#FBbi_00000346");

        Set<Atom> setOfAtoms = new HashSet<>();
        setOfAtoms.add(new ClassAtom(detectionOfPhotons, x0));
        setOfAtoms.add(new ClassAtom(polarizationSensitiveDetection, x1));
        setOfAtoms.add(new RoleAtom(makesUseOf, x0, x1));
        setOfAtoms.add(new RoleAtom(makesUseOf, x1, x0));
        Set<Variable> avarSet = Sets.newHashSet(x0);
        RootedCQ q0 = new RootedCQ(avarSet, setOfAtoms);

        QueryCaller.runRootedCQ(q0, Thread.currentThread().getStackTrace()[1].getMethodName(), "fbbi06");
    }

    public static void runFBbi_coagulationFixativeFixedTissue() throws CQException, Exception {
        Signature sigma = CQContext.getINSTANCE().getOriginalSignature();

        Variable x0 = new Variable(0);
        Variable x1 = new Variable(1);
        Variable x2 = new Variable(2);

        OWLClass microscopyWithLenses = sigma.getCls("#FBbi_00000343");
        OWLClass detectionOfElectrons = sigma.getCls("#FBbi_00000375");
        OWLClass coagulationFixativeFixedTissue = sigma.getCls("#FBbi_00000003");
        OWLObjectProperty makesUseOf = sigma.getRole("#FBbi_00000346");

        Set<Atom> setOfAtoms = new HashSet<>();
        setOfAtoms.add(new ClassAtom(microscopyWithLenses, x0));
        setOfAtoms.add(new ClassAtom(detectionOfElectrons, x1));
        setOfAtoms.add(new ClassAtom(coagulationFixativeFixedTissue, x2));
        setOfAtoms.add(new RoleAtom(makesUseOf, x0, x2));
        setOfAtoms.add(new RoleAtom(makesUseOf, x1, x2));
        Set<Variable> avarSet = Sets.newHashSet(x0);
        RootedCQ q0 = new RootedCQ(avarSet, setOfAtoms);

        QueryCaller.runRootedCQ(q0, Thread.currentThread().getStackTrace()[1].getMethodName(), "fbbi07");
    }

    public static void runFBbi_arrayScanConfocal() throws CQException, Exception {
        Signature sigma = CQContext.getINSTANCE().getOriginalSignature();

        Variable x0 = new Variable(0);
        Variable x1 = new Variable(1);
        Variable x2 = new Variable(2);

        OWLClass microscopy = sigma.getCls("#FBbi_00000241");
        OWLClass arrayScanConfocal = sigma.getCls("#FBbi_00000393");
        OWLClass handSectionedTissue = sigma.getCls("#FBbi_00000028");
        OWLObjectProperty makesUseOf = sigma.getRole("#FBbi_00000346");

        Set<Atom> setOfAtoms = new HashSet<>();
        setOfAtoms.add(new ClassAtom(microscopy, x0));
        setOfAtoms.add(new ClassAtom(arrayScanConfocal, x1));
        setOfAtoms.add(new ClassAtom(handSectionedTissue, x2));
        setOfAtoms.add(new RoleAtom(makesUseOf, x0, x2));
        setOfAtoms.add(new RoleAtom(makesUseOf, x1, x2));
        Set<Variable> avarSet = Sets.newHashSet(x0);
        RootedCQ q0 = new RootedCQ(avarSet, setOfAtoms);

        QueryCaller.runRootedCQ(q0, Thread.currentThread().getStackTrace()[1].getMethodName(), "fbbi10");
    }

    public static void runFBbi_opticalHighlighter() throws CQException, Exception {
        Signature sigma = CQContext.getINSTANCE().getOriginalSignature();

        Variable x0 = new Variable(0);
        Variable x1 = new Variable(1);
        Variable x2 = new Variable(2);
        Variable x3 = new Variable(3);
        Variable x4 = new Variable(4);

        OWLClass tissueInPlasticEmbedment = sigma.getCls("#FBbi_00000016");
        OWLClass macromolecularProbe = sigma.getCls("#FBbi_00000410");
        OWLClass saponinPermeabilized = sigma.getCls("#FBbi_00000264");
        OWLClass opticalHighlighter = sigma.getCls("#FBbi_00000434");
        OWLClass lectin = sigma.getCls("#FBbi_00000402");
        OWLObjectProperty makesUseOf = sigma.getRole("#FBbi_00000346");

        Set<Atom> setOfAtoms = new HashSet<>();
        setOfAtoms.add(new ClassAtom(tissueInPlasticEmbedment, x0));
        setOfAtoms.add(new ClassAtom(macromolecularProbe, x1));
        setOfAtoms.add(new ClassAtom(saponinPermeabilized, x2));
        setOfAtoms.add(new ClassAtom(opticalHighlighter, x3));
        setOfAtoms.add(new ClassAtom(lectin, x4));
        setOfAtoms.add(new RoleAtom(makesUseOf, x0, x3));
        setOfAtoms.add(new RoleAtom(makesUseOf, x1, x3));
        setOfAtoms.add(new RoleAtom(makesUseOf, x1, x4));
        setOfAtoms.add(new RoleAtom(makesUseOf, x2, x4));
        Set<Variable> avarSet = Sets.newHashSet(x1);
        RootedCQ q0 = new RootedCQ(avarSet, setOfAtoms);

        QueryCaller.runRootedCQ(q0, Thread.currentThread().getStackTrace()[1].getMethodName(), "fbbi08");
    }

    public static void runFBbi_greyScaleGraphic() throws CQException, Exception {
        Signature sigma = CQContext.getINSTANCE().getOriginalSignature();

        Variable x0 = new Variable(0);
        Variable x1 = new Variable(1);
        Variable x2 = new Variable(2);

        OWLClass greyScaleGraphic = sigma.getCls("#FBbi_00000227");
        OWLClass dispersedCellsInVitro = sigma.getCls("#FBbi_00000611");
        OWLClass spinningDiscConfocalMicroscopy = sigma.getCls("#FBbi_00000253");
        OWLObjectProperty makesUseOf = sigma.getRole("#FBbi_00000346");

        Set<Atom> setOfAtoms = new HashSet<>();
        setOfAtoms.add(new ClassAtom(greyScaleGraphic, x0));
        setOfAtoms.add(new ClassAtom(dispersedCellsInVitro, x1));
        setOfAtoms.add(new ClassAtom(spinningDiscConfocalMicroscopy, x2));
        setOfAtoms.add(new RoleAtom(makesUseOf, x0, x1));
        setOfAtoms.add(new RoleAtom(makesUseOf, x0, x2));
        Set<Variable> avarSet = Sets.newHashSet(x0);
        RootedCQ q0 = new RootedCQ(avarSet, setOfAtoms);

        QueryCaller.runRootedCQ(q0, Thread.currentThread().getStackTrace()[1].getMethodName(), "fbbi09");
    }

    public static void runMOHSE_endocarditis() throws CQException, Exception {
        Signature sigma = CQContext.getINSTANCE().getOriginalSignature();

        Variable x0 = new Variable(0);
        Variable x1 = new Variable(1);

        OWLClass organism = sigma.getCls("#Organism");
        OWLClass endocarditis = sigma.getCls("#Endocarditis");
        OWLObjectProperty shows = sigma.getRole("shows");

        Set<Atom> setOfAtoms = new HashSet<>();
        setOfAtoms.add(new ClassAtom(organism, x0));
        setOfAtoms.add(new ClassAtom(endocarditis, x1));
        setOfAtoms.add(new RoleAtom(shows, x0, x1));
        Set<Variable> avarSet = Sets.newHashSet(x0, x1);
        AvarCQ q0 = new AvarCQ(avarSet, setOfAtoms);

        QueryCaller.runAvarCQ(q0, Thread.currentThread().getStackTrace()[1].getMethodName(), "mohse01");
    }

    public static void runMOHSE_tissue() throws CQException, Exception {
        Signature sigma = CQContext.getINSTANCE().getOriginalSignature();

        Variable x0 = new Variable(0);
        Variable x1 = new Variable(1);
        Variable x2 = new Variable(2);
        Variable x3 = new Variable(3);

        OWLClass organism = sigma.getCls("#Organism");
        OWLClass surgery = sigma.getCls("#CardiacSurgery");
        OWLClass tissue = sigma.getCls("#Tissue");
        OWLClass internalOrgan = sigma.getCls("#InternalOrgan");
        OWLObjectProperty hasPersonPerforming = sigma.getRole("hasCausalAgent");
        OWLObjectProperty actsOn = sigma.getRole("actsOn");
        OWLObjectProperty hasAlphaConnection = sigma.getRole("hasAlphaConnection");

        Set<Atom> setOfAtoms = new HashSet<>();
        setOfAtoms.add(new ClassAtom(organism, x0));
        setOfAtoms.add(new ClassAtom(surgery, x1));
        setOfAtoms.add(new ClassAtom(tissue, x2));
        setOfAtoms.add(new ClassAtom(internalOrgan, x3));
        setOfAtoms.add(new RoleAtom(hasPersonPerforming, x1, x0));
        setOfAtoms.add(new RoleAtom(actsOn, x1, x2));
        setOfAtoms.add(new RoleAtom(actsOn, x1, x3));
        setOfAtoms.add(new RoleAtom(hasAlphaConnection, x2, x3));
        Set<Variable> avarSet = Sets.newHashSet(x0, x1, x2, x3);
        AvarCQ q0 = new AvarCQ(avarSet, setOfAtoms);

        QueryCaller.runAvarCQ(q0, Thread.currentThread().getStackTrace()[1].getMethodName(), "mohse02");
    }

    public static void runMOHSE_alcohol() throws CQException, Exception {
        Signature sigma = CQContext.getINSTANCE().getOriginalSignature();

        Variable x0 = new Variable(0);
        Variable x1 = new Variable(1);
        Variable x2 = new Variable(2);
        Variable x3 = new Variable(3);

        OWLClass triglyceride = sigma.getCls("#Triglyceride");
        OWLClass alcohol = sigma.getCls("#Alcohol");
        OWLClass investigationAct = sigma.getCls("#InvestigationAct");
        OWLClass careAct = sigma.getCls("#CareAct");
        OWLObjectProperty serves = sigma.getRole("serves");
        OWLObjectProperty hasDissolvedWithin = sigma.getRole("hasDissolvedWithin");

        Set<Atom> setOfAtoms = new HashSet<>();
        setOfAtoms.add(new ClassAtom(triglyceride, x0));
        setOfAtoms.add(new ClassAtom(alcohol, x1));
        setOfAtoms.add(new ClassAtom(investigationAct, x2));
        setOfAtoms.add(new ClassAtom(careAct, x3));
        setOfAtoms.add(new RoleAtom(hasDissolvedWithin, x0, x1));
        setOfAtoms.add(new RoleAtom(serves, x0, x2));
        setOfAtoms.add(new RoleAtom(serves, x1, x3));
        Set<Variable> avarSet = Sets.newHashSet(x0, x1);
        TqCQ q0 = new TqCQ(avarSet, setOfAtoms);

        QueryCaller.runTqCQ(q0, Thread.currentThread().getStackTrace()[1].getMethodName(), "mohse03");
    }

    public static void runMOHSE_giardia() throws CQException, Exception {
        Signature sigma = CQContext.getINSTANCE().getOriginalSignature();

        Variable x0 = new Variable(0);
        Variable x1 = new Variable(1);

        OWLClass aminoglycoside = sigma.getCls("#Aminoglycoside");
        OWLClass giardia = sigma.getCls("#Giardia");
        OWLObjectProperty actsOn = sigma.getRole("actsOn");

        Set<Atom> setOfAtoms = new HashSet<>();
        setOfAtoms.add(new ClassAtom(aminoglycoside, x0));
        setOfAtoms.add(new ClassAtom(giardia, x1));
        setOfAtoms.add(new RoleAtom(actsOn, x0, x1));
        Set<Variable> avarSet = Sets.newHashSet(x1);
        RootedCQ q0 = new RootedCQ(avarSet, setOfAtoms);

        QueryCaller.runRootedCQ(q0, Thread.currentThread().getStackTrace()[1].getMethodName(), "mohse04");
    }

    public static void runMOHSE_duct() throws CQException, Exception {
        Signature sigma = CQContext.getINSTANCE().getOriginalSignature();

        Variable x0 = new Variable(0);
        Variable x1 = new Variable(1);

        OWLClass Duct = sigma.getCls("#Duct");
        OWLClass InternalOrgan = sigma.getCls("#InternalOrgan");
        OWLObjectProperty isBlindPouchDivisionOf = sigma.getRole("isBlindPouchDivisionOf");
        OWLObjectProperty carriesTo = sigma.getRole("carriesTo");

        Set<Atom> setOfAtoms = new HashSet<>();
        setOfAtoms.add(new ClassAtom(Duct, x0));
        setOfAtoms.add(new ClassAtom(InternalOrgan, x1));
        setOfAtoms.add(new RoleAtom(isBlindPouchDivisionOf, x0, x1));
        setOfAtoms.add(new RoleAtom(carriesTo, x0, x1));
        Set<Variable> avarSet = Sets.newHashSet(x0);
        RootedCQ q0 = new RootedCQ(avarSet, setOfAtoms);

        QueryCaller.runRootedCQ(q0, Thread.currentThread().getStackTrace()[1].getMethodName(), "mohse05");
    }

    public static void runMOHSE_voluntaryMovement() throws CQException, Exception {
        Signature sigma = CQContext.getINSTANCE().getOriginalSignature();

        Variable x0 = new Variable(0);
        Variable x1 = new Variable(1);

        OWLClass pathologyConstruct = sigma.getCls("#PathologyConstruct");
        OWLClass voluntaryMovement = sigma.getCls("#VoluntaryMovement");
        OWLObjectProperty isCausalAgentOf = sigma.getRole("isCausalAgentOf");
        OWLObjectProperty occursDuring = sigma.getRole("occursDuring");

        Set<Atom> setOfAtoms = new HashSet<>();
        setOfAtoms.add(new ClassAtom(pathologyConstruct, x0));
        setOfAtoms.add(new ClassAtom(voluntaryMovement, x1));
        setOfAtoms.add(new RoleAtom(isCausalAgentOf, x0, x0));
        setOfAtoms.add(new RoleAtom(occursDuring, x0, x1));
        Set<Variable> avarSet = Sets.newHashSet(x1);
        RootedCQ q0 = new RootedCQ(avarSet, setOfAtoms);

        QueryCaller.runRootedCQ(q0, Thread.currentThread().getStackTrace()[1].getMethodName(), "mohse06");
    }

    public static void runMOHSE_tetracycline() throws CQException, Exception {
        Signature sigma = CQContext.getINSTANCE().getOriginalSignature();

        Variable x0 = new Variable(0);
        Variable x1 = new Variable(1);
        Variable x2 = new Variable(2);

        OWLClass protein = sigma.getCls("#Protein");
        OWLClass internalOrgan = sigma.getCls("#InternalOrgan");
        OWLClass Tetracycline = sigma.getCls("#Tetracycline");
        OWLObjectProperty isActedOnSpecificallyBy = sigma.getRole("isActedOnSpecificallyBy");
        OWLObjectProperty contains = sigma.getRole("contains");

        Set<Atom> setOfAtoms = new HashSet<>();
        setOfAtoms.add(new ClassAtom(protein, x0));
        setOfAtoms.add(new ClassAtom(internalOrgan, x1));
        setOfAtoms.add(new ClassAtom(Tetracycline, x2));
        setOfAtoms.add(new RoleAtom(contains, x0, x2));
        setOfAtoms.add(new RoleAtom(isActedOnSpecificallyBy, x1, x2));
        Set<Variable> avarSet = Sets.newHashSet(x0);
        RootedCQ q0 = new RootedCQ(avarSet, setOfAtoms);

        QueryCaller.runRootedCQ(q0, Thread.currentThread().getStackTrace()[1].getMethodName(), "mohse07");
    }

    public static void runMOHSE_teichoicAcid() throws CQException, Exception {
        Signature sigma = CQContext.getINSTANCE().getOriginalSignature();

        Variable x0 = new Variable(0);
        Variable x1 = new Variable(1);
        Variable x2 = new Variable(2);
        Variable x3 = new Variable(3);
        Variable x4 = new Variable(4);

        OWLClass sulphonamide = sigma.getCls("#Sulphonamide");
        OWLClass enzyme = sigma.getCls("#NAMEDEnzyme");
        OWLClass teichoicAcid = sigma.getCls("#TeichoicAcid");
        OWLClass tumorMarkerRole = sigma.getCls("#Tumour");
        OWLClass liver = sigma.getCls("#Liver");
        OWLObjectProperty serves = sigma.getRole("serves");
        OWLObjectProperty actsOn = sigma.getRole("actsOn");

        Set<Atom> setOfAtoms = new HashSet<>();
        setOfAtoms.add(new ClassAtom(sulphonamide, x0));
        setOfAtoms.add(new ClassAtom(enzyme, x1));
        setOfAtoms.add(new ClassAtom(teichoicAcid, x2));
        setOfAtoms.add(new ClassAtom(tumorMarkerRole, x3));
        setOfAtoms.add(new ClassAtom(liver, x4));
        setOfAtoms.add(new RoleAtom(serves, x0, x3));
        setOfAtoms.add(new RoleAtom(serves, x1, x3));
        setOfAtoms.add(new RoleAtom(actsOn, x1, x4));
        setOfAtoms.add(new RoleAtom(actsOn, x2, x4));
        Set<Variable> avarSet = Sets.newHashSet(x1);
        RootedCQ q0 = new RootedCQ(avarSet, setOfAtoms);

        QueryCaller.runRootedCQ(q0, Thread.currentThread().getStackTrace()[1].getMethodName(), "mohse08");
    }

    public static void runMOHSE_bordatella() throws CQException, Exception {
        Signature sigma = CQContext.getINSTANCE().getOriginalSignature();

        Variable x0 = new Variable(0);
        Variable x1 = new Variable(1);
        Variable x2 = new Variable(2);

        OWLClass arterialDisease = sigma.getCls("#ArterialDisease");
        OWLClass bordatella = sigma.getCls("#Bordatella");
        OWLClass anonymous294 = sigma.getCls("#Anonymous-294");
        OWLObjectProperty hasCausalAgent = sigma.getRole("hasCausalAgent");
        OWLObjectProperty isCausallyLinkedTo = sigma.getRole("IsCausallyLinkedTo");

        Set<Atom> setOfAtoms = new HashSet<>();
        setOfAtoms.add(new ClassAtom(arterialDisease, x0));
        setOfAtoms.add(new ClassAtom(bordatella, x1));
        setOfAtoms.add(new ClassAtom(anonymous294, x2));
        setOfAtoms.add(new RoleAtom(hasCausalAgent, x0, x1));
        setOfAtoms.add(new RoleAtom(isCausallyLinkedTo, x0, x2));
        Set<Variable> avarSet = Sets.newHashSet(x0);
        RootedCQ q0 = new RootedCQ(avarSet, setOfAtoms);

        QueryCaller.runRootedCQ(q0, Thread.currentThread().getStackTrace()[1].getMethodName(), "mohse09");
    }

    public static void runMOHSE_brain() throws CQException, Exception {
        Signature sigma = CQContext.getINSTANCE().getOriginalSignature();

        Variable x0 = new Variable(0);
        Variable x1 = new Variable(1);
        Variable x2 = new Variable(2);

        OWLClass investigationAct = sigma.getCls("#InvestigationAct");
        OWLClass anonymous294 = sigma.getCls("#Anonymous-294");
        OWLClass brain = sigma.getCls("#Brain");
        OWLObjectProperty actsOn = sigma.getRole("actsOn");

        Set<Atom> setOfAtoms = new HashSet<>();
        setOfAtoms.add(new ClassAtom(investigationAct, x0));
        setOfAtoms.add(new ClassAtom(anonymous294, x1));
        setOfAtoms.add(new ClassAtom(brain, x2));
        setOfAtoms.add(new RoleAtom(actsOn, x0, x2));
        setOfAtoms.add(new RoleAtom(actsOn, x1, x2));
        Set<Variable> avarSet = Sets.newHashSet(x0);
        RootedCQ q0 = new RootedCQ(avarSet, setOfAtoms);

        QueryCaller.runRootedCQ(q0, Thread.currentThread().getStackTrace()[1].getMethodName(), "mohse10");
    }

    */

}
