/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package de.unibremen.informatik.tdki.grind2.syntax;

import de.unibremen.informatik.tdki.grind2.context.CQContext;
import de.unibremen.informatik.tdki.grind2.cq.CQ;
import de.unibremen.informatik.tdki.grind2.cq.CQException;
import de.unibremen.informatik.tdki.grind2.cq.components.Atom;
import de.unibremen.informatik.tdki.grind2.cq.components.ClassAtom;
import de.unibremen.informatik.tdki.grind2.cq.components.RoleAtom;
import de.unibremen.informatik.tdki.grind2.cq.components.Variable;
import de.unibremen.informatik.tdki.grind2.utilities.Signature;
import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.nio.charset.Charset;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import org.semanticweb.owlapi.model.OWLClass;
import org.semanticweb.owlapi.model.OWLObjectProperty;

/**
 *
 * @author hansen
 */
public class CQParser {

    private final List<String> listOfLines;
    private final Signature sigma;

    public CQParser(Signature sigma) throws IOException {
        this.sigma = sigma;
        this.listOfLines = new ArrayList<>();
    }

    public void readFile(String filename) throws IOException {
        String line;
        try (InputStream fis = new FileInputStream(filename);
                InputStreamReader isr = new InputStreamReader(fis, Charset.forName("UTF-8"));
                BufferedReader br = new BufferedReader(isr);) {
            while ((line = br.readLine()) != null) {
                readLine(line);
            }
        }
    }

    public void readLine(String line) {
        line = line.replaceAll("\\s", "");
        if (line.startsWith("#")) {
            return;
        }
        if (line.startsWith("'")) {
            line = line.substring(1);
            if (line.endsWith("'")) {
                line = line.substring(0, line.length() - 1);
            }
        }
        if (line.isEmpty()) {
            return;
        }
        this.listOfLines.add(line);
    }

    public List<CQ> getListOfCQs() throws SyntaxException, CQException {
        List<CQ> listOfCQs = new ArrayList<>();

        int i = 1;
        for (String line : this.listOfLines) {
            String[] parts = line.split("<-");
            if (parts.length == 1) {
                throw new SyntaxException("[" + this.getClass().getSimpleName()
                        + "] Malformed query, line " + i + ": No '<-' !");
            }else if (parts.length > 2) {
                throw new SyntaxException("[" + this.getClass().getSimpleName()
                        + "] Malformed query, line " + i + ": Too many '<-' !");
            }

            // Process head "Q(?0,?1,?2)"
            String headString = parts[0].replaceFirst("Q\\(", "").replaceFirst("\\)", "").replaceAll("\\?", "");
            String[] avarIdxArray = headString.split(",");

            Set<Variable> avars = new HashSet<>();
            for (String idxString : avarIdxArray) {
                int index = Integer.parseInt(idxString);
                Variable x = new Variable(index);
                avars.add(x);
            }

            // Process body "owlENVO_00002272(?0),owlENVO_located_in(?0,?1),owlENVO_00000358(?1)"
            Set<Atom> setOfAtoms = new HashSet<>();

            String[] atomArray = parts[1].split("\\)");
            for (String atomString : atomArray) {
                if (atomString.startsWith(",")) {
                    atomString = atomString.replaceFirst(",", "");
                }
                String[] cnAndVars = atomString.split("\\(");
                if (cnAndVars.length != 2) {
                    throw new SyntaxException("[" + this.getClass().getSimpleName()
                            + "] Malformed query, line " + i + ": Cannot split body!");
                }
                String symbolString = cnAndVars[0];

                String varsString = cnAndVars[1].replaceAll("\\?", "");
                String[] varsArray = varsString.split(",");
                switch (varsArray.length) {
                    case 1: {
                        try {
                            OWLClass cls = this.sigma.getCls(symbolString);
                            int indexX = Integer.parseInt(varsArray[0]);
                            Variable x = new Variable(indexX);
                            setOfAtoms.add(new ClassAtom(cls, x));
                        } catch (CQException e) {
                            System.err.println(e.getMessage());
                            System.exit(-1);
                        }
                        break;
                    }
                    case 2: {
                        try {
                            OWLObjectProperty role = this.sigma.getRole(symbolString);
                            int indexX = Integer.parseInt(varsArray[0]);
                            int indexY = Integer.parseInt(varsArray[1]);
                            Variable x = new Variable(indexX);
                            Variable y = new Variable(indexY);
                            setOfAtoms.add(new RoleAtom(role, x, y));
                        } catch (CQException e) {
                            System.err.println(e.getMessage());
                            System.exit(-1);
                        }
                        break;
                    }
                    default: {
                        throw new SyntaxException("[" + this.getClass().getSimpleName()
                                + "] Malformed query, line " + i + ": Number of variables is neither 1 nor 2!");
                    }
                }
            }
            CQ cq = new CQ(avars, setOfAtoms);
            listOfCQs.add(cq);

            i++; // line increase
        } // lines
        return listOfCQs;
    }
}
