/*
 * Copyright 2014, 2015 Peter Hansen and İnanç Seylan,
 *           2016, 2017 Peter Hansen.
 * 
 * This file is part of Grind v2.
 * 
 * Grind is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Grind is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Grind.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.unibremen.informatik.tdki.grind2.utilities;

import de.unibremen.informatik.tdki.grind2.cq.components.Variable;
import java.util.HashSet;
import java.util.Set;
import org.semanticweb.owlapi.model.OWLClass;
import org.semanticweb.owlapi.model.OWLObjectProperty;

/**
 *
 * @author hansen
 */
public class ConceptTreeNode { // implements Iterable<TreeNode<T>> {

    Variable x;
    Set<OWLClass> setOfClasses;
    ConceptTreeNode parent;
    OWLObjectProperty reachingRole;
    Set<ConceptTreeNode> children;

    public ConceptTreeNode(Variable x) {
        this.x = x;
        this.setOfClasses = new HashSet<>();
        this.children = new HashSet<>();
        this.reachingRole = null;
    }

    public void addClass(OWLClass cls) {
        this.setOfClasses.add(cls);
    }

    public void addChild(ConceptTreeNode y, OWLObjectProperty role) {
        y.parent = this;
        y.reachingRole = role;
        this.children.add(y);
    }

    public Set<ConceptTreeNode> getChildren() {
        return children;
    }

    public boolean isLeaf() {
        return children.isEmpty();
    }

    public Set<OWLClass> getSetOfClasses() {
        return setOfClasses;
    }

    public OWLObjectProperty getReachingRole() {
        return reachingRole;
    }

    public Variable getX() {
        return x;
    }

//    public void addParent(ConceptTreeNode z) {
//        this.parent = z;
//    }
    @Override
    public String toString() {
        String result = "(node=" + x + ", classes=" + setOfClasses + ", reachingRole=" + reachingRole + ", children=" + children + ")";
        return result;
    }
}
