/*
 * Copyright 2014, 2015 Peter Hansen and İnanç Seylan,
 *           2016, 2017 Peter Hansen.
 * 
 * This file is part of Grind v2.
 * 
 * Grind is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Grind is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Grind.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.unibremen.informatik.tdki.grind2.utilities;

import de.unibremen.informatik.tdki.grind.utilities.ELUtils;
import de.unibremen.informatik.tdki.grind2.cq.CQException;
import java.util.HashSet;
import java.util.Set;
import org.semanticweb.owlapi.model.OWLAxiom;
import org.semanticweb.owlapi.model.OWLClass;
import org.semanticweb.owlapi.model.OWLObjectProperty;
import org.semanticweb.owlapi.model.OWLSubClassOfAxiom;

/**
 *
 * @author hansen
 */
public class Signature {

    Set<OWLClass> setOfClasses;
    Set<OWLObjectProperty> setOfRoles;

    public Signature(Set<OWLClass> setOfClasses, Set<OWLObjectProperty> setOfRoles) {
        this.setOfClasses = setOfClasses;
        this.setOfRoles = setOfRoles;
    }

    public static Signature getSignatureForAxioms(Set<OWLAxiom> axioms) {
        Set<OWLSubClassOfAxiom> scAxioms = new HashSet<>();
        for (OWLAxiom ax : axioms) {
            OWLSubClassOfAxiom scax = (OWLSubClassOfAxiom) ax;
            scAxioms.add(scax);
        }
        Set<OWLClass> classes = ELUtils.getConceptNames(scAxioms);
        Set<OWLObjectProperty> roles = ELUtils.getRoleNames(scAxioms);
        return new Signature(classes, roles);
    }
    
    public Set<OWLClass> getSetOfClasses() {
        return setOfClasses;
    }

    public Set<OWLObjectProperty> getSetOfRoles() {
        return setOfRoles;
    }

    public OWLClass getCls(String searchString) throws CQException {
        for (OWLClass cls : this.setOfClasses) {
            if (cls.getIRI().getFragment() == null) {
                continue;
            }
//            System.out.println(cls.getIRI());
            if (cls.getIRI().toString().endsWith(searchString)) {
                return cls;
            }
        }
        throw new CQException("[" + this.getClass().getSimpleName() + "] Could not find concept name ``" + searchString + "'' in ontology!");
    }

    public OWLObjectProperty getRole(String searchString) throws CQException {
        for (OWLObjectProperty role : this.setOfRoles) {
//            System.out.println(role.getIRI());
            if (role.getIRI().toString().endsWith(searchString)) {
                return role;
            }
        }
        throw new CQException("[" + this.getClass().getSimpleName() + "] Could not find role name ``" + searchString + "'' in ontology!");
    }

    @Override
    public String toString() {
        return "Signature{" + "setOfClasses=" + setOfClasses + ", setOfRoles=" + setOfRoles + '}';
    }

}
