/*
 * Copyright 2014, 2015 Peter Hansen and İnanç Seylan,
 *           2016, 2017 Peter Hansen.
 * 
 * This file is part of Grind v2.
 * 
 * Grind is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Grind is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Grind.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.unibremen.informatik.tdki.grind2.utilities;

import java.io.File;
import java.util.Set;
import org.semanticweb.owlapi.apibinding.OWLManager;
import org.semanticweb.owlapi.model.AxiomType;
import org.semanticweb.owlapi.model.OWLClass;
import org.semanticweb.owlapi.model.OWLObjectProperty;
import org.semanticweb.owlapi.model.OWLOntology;
import org.semanticweb.owlapi.model.OWLOntologyManager;
import org.semanticweb.owlapi.model.OWLSubClassOfAxiom;

/**
 *
 * @author hansen
 */
public class SimpleOntology {

    Set<OWLSubClassOfAxiom> setOfCIs;
    Signature sigma;

    public SimpleOntology() {
    }

    public void loadFromFile(String fileName) {
        OWLOntologyManager manager;
        File file;
        OWLOntology ont = null;

        manager = OWLManager.createOWLOntologyManager();
        file = new File(fileName);
        try {
            ont = manager.loadOntologyFromOntologyDocument(file);
        } catch (Exception e) {
            System.err.println("Error: Could not load ontology: " + e);
            System.exit(-1);
        }
        manager.removeOntology(ont);
        
        setOfCIs = ont.getAxioms(AxiomType.SUBCLASS_OF);
        Set<OWLClass> setOfClasses = ont.getClassesInSignature();
        Set<OWLObjectProperty> setOfRoles = ont.getObjectPropertiesInSignature();
        sigma = new Signature(setOfClasses, setOfRoles);
    }

    public Set<OWLSubClassOfAxiom> getSetOfCIs() {
        return setOfCIs;
    }

    public Signature getSignature() {
        return sigma;
    }

    @Override
    public String toString() {
        return setOfCIs.toString();
    }

}
